# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class LarterBreakspear(ComponentWithWidget):
    gCa: InArg[float]
    gK: InArg[float]
    gL: InArg[float]
    phi: InArg[float]
    gNa: InArg[float]
    TK: InArg[float]
    TCa: InArg[float]
    TNa: InArg[float]
    VCa: InArg[float]
    VK: InArg[float]
    VL: InArg[float]
    VNa: InArg[float]
    d_K: InArg[float]
    tau_K: InArg[float]
    d_Na: InArg[float]
    d_Ca: InArg[float]
    aei: InArg[float]
    aie: InArg[float]
    b: InArg[float]
    C: InArg[float]
    ane: InArg[float]
    ani: InArg[float]
    aee: InArg[float]
    Iext: InArg[float]
    rNMDA: InArg[float]
    VT: InArg[float]
    d_V: InArg[float]
    ZT: InArg[float]
    d_Z: InArg[float]
    QV_max: InArg[float]
    QZ_max: InArg[float]
    t_scale: InArg[float]
    variables_of_interest: InArg[list]

    larterBreakspear: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.larter_breakspear import LarterBreakspear
        return LarterBreakspear

    def execute(self, ctx) -> None:
        larterBreakspear = self.tvb_ht_class()

        set_values(self, larterBreakspear)
        self.larterBreakspear.value = larterBreakspear
        print_component_summary(self.larterBreakspear.value)



from tqdm import tqdm
import os
from urllib import request, parse
from github import Github


def request_folder(folder, repo_name="XpressAi/Xircuits", branch="master"):
    print("Downloading " + folder + " from " + repo_name + " branch " + branch)
    g = Github()
    
    try:
        repo = g.get_repo(repo_name)
        contents = repo.get_contents(folder, ref=branch)
    except:
       print(folder + " from " + repo_name + " branch " + branch + " does not exist!")
       return 

    if not os.path.exists(folder):
        os.mkdir(folder)
    else:
        print(folder + " already exists.")
    
    base_url = "https://raw.githubusercontent.com/" + repo_name + "/" + branch    
    urls = {}
    
    while len(contents)>0:
        file_content = contents.pop(0)
        if file_content.type=='dir':
            if not os.path.exists(file_content.path):
                os.mkdir(file_content.path)
            contents.extend(repo.get_contents(file_content.path, ref=branch))

        else:
            file_url = base_url + "/" + parse.quote(file_content.path)
            urls.update({file_url: file_content.path})

    for url in tqdm(urls):
        try:
            request.urlretrieve(url, urls[url])
        except:
            print("Unable to retrieve " + urls[url] + ". Skipping...")


# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

import os
from siibra.retrieval import SiibraHttpRequestError
from tvb.adapters.creators import siibra_base as sb
from tvb.datatypes.connectivity import Connectivity
from tvbwidgets.core.auth import get_current_token
from xai_components.base import InArg, OutArg, Component, xai_component
from xai_components.base_tvb import TVBComponent
from xai_components.utils import print_component_summary


@xai_component(color='rgb(85, 37, 130)')
class ConnectivityFromFile(TVBComponent):
    file_path: InArg[str]

    connectivity: OutArg[Connectivity]

    def __init__(self):
        self.done = False
        self.file_path = InArg(None)
        self.connectivity = OutArg(None)

    def execute(self, ctx) -> None:
        # imports
        from matplotlib import pyplot as plt
        from tvb.datatypes.connectivity import Connectivity

        file_path = self.file_path.value
        if not file_path:
            file_path = 'connectivity_76.zip'  # default from tvb_data
        connectivity = Connectivity.from_file(file_path)
        connectivity.configure()

        self.connectivity.value = connectivity
        print_component_summary(self.connectivity.value)

        plt.imshow(self.connectivity.value.weights, interpolation='none')
        plt.show()


@xai_component(color='rgb(85, 37, 130)')
class ConnectivityFromSiibra(Component):
    atlas: InArg[str]
    parcellation: InArg[str]
    subject_id: InArg[str]

    connectivity: OutArg[Connectivity]

    def __init__(self):
        self.done = False
        self.atlas = InArg(sb.DEFAULT_ATLAS)
        self.parcellation = InArg(sb.DEFAULT_PARCELLATION)
        self.subject_id = InArg('000')
        self.connectivity = OutArg(None)

    def execute(self, ctx) -> None:
        # imports
        from matplotlib import pyplot as plt

        atlas = self.atlas.value
        parcellation = self.parcellation.value
        subject_id = self.subject_id.value

        token = get_current_token()
        os.environ['HBP_AUTH_TOKEN'] = token

        try:
            sc_dict, _ = sb.get_connectivities_from_kg(atlas=atlas, parcellation=parcellation,
                                                       subject_ids=subject_id,
                                                       compute_fc=False)
            connectivity = sc_dict[subject_id]
            connectivity.configure()

            self.connectivity.value = connectivity
            print_component_summary(self.connectivity.value)

            plt.imshow(self.connectivity.value.weights, interpolation='none')
            plt.show()
        except SiibraHttpRequestError as e:
            if e.response.status_code in [401, 403]:
                raise ConnectionError('Invalid EBRAINS authentication token. Please provide a new one as environment '
                                      'variable HBP_AUTH_TOKEN.')
            else:
                raise ConnectionError('We could not complete the operation. '
                                      'Please check the logs and contact the development team from TVB, siibra or EBRAINS KG.')



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class Epileptor(ComponentWithWidget):
    a: InArg[float]
    b: InArg[float]
    c: InArg[float]
    d: InArg[float]
    r: InArg[float]
    s: InArg[float]
    x0: InArg[float]
    Iext: InArg[float]
    slope: InArg[float]
    Iext2: InArg[float]
    tau: InArg[float]
    aa: InArg[float]
    bb: InArg[float]
    Kvf: InArg[float]
    Kf: InArg[float]
    Ks: InArg[float]
    tt: InArg[float]
    modification: InArg[bool]
    variables_of_interest: InArg[list]

    epileptor: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.epileptor import Epileptor
        return Epileptor

    def execute(self, ctx) -> None:
        epileptor = self.tvb_ht_class()

        set_values(self, epileptor)
        self.epileptor.value = epileptor
        print_component_summary(self.epileptor.value)


@xai_component(color='rgb(101, 179, 46)')
class Epileptor2D(ComponentWithWidget):
    a: InArg[float]
    b: InArg[float]
    c: InArg[float]
    d: InArg[float]
    r: InArg[float]
    x0: InArg[float]
    Iext: InArg[float]
    slope: InArg[float]
    Kvf: InArg[float]
    Ks: InArg[float]
    tt: InArg[float]
    modification: InArg[bool]
    variables_of_interest: InArg[list]

    epileptor2D: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.epileptor import Epileptor2D
        return Epileptor2D

    def execute(self, ctx) -> None:
        epileptor2D = self.tvb_ht_class()

        set_values(self, epileptor2D)
        self.epileptor2D.value = epileptor2D
        print_component_summary(self.epileptor2D.value)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class EpileptorRestingState(ComponentWithWidget):
    a: InArg[float]
    b: InArg[float]
    c: InArg[float]
    d: InArg[float]
    r: InArg[float]
    s: InArg[float]
    x0: InArg[float]
    Iext: InArg[float]
    slope: InArg[float]
    Iext2: InArg[float]
    tau: InArg[float]
    aa: InArg[float]
    bb: InArg[float]
    Kvf: InArg[float]
    Kf: InArg[float]
    Ks: InArg[float]
    tt: InArg[float]
    tau_rs: InArg[float]
    I_rs: InArg[float]
    a_rs: InArg[float]
    b_rs: InArg[float]
    d_rs: InArg[float]
    e_rs: InArg[float]
    f_rs: InArg[float]
    alpha_rs: InArg[float]
    beta_rs: InArg[float]
    gamma_rs: InArg[float]
    K_rs: InArg[float]
    p: InArg[float]
    variables_of_interest: InArg[list]

    epileptorRestingState: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.epileptor_rs import EpileptorRestingState
        return EpileptorRestingState

    def execute(self, ctx) -> None:
        epileptorRestingState = self.tvb_ht_class()

        set_values(self, epileptorRestingState)
        self.epileptorRestingState.value = epileptorRestingState
        print_component_summary(self.epileptorRestingState.value)



import os.path
import shutil

from tvbextxircuits import nb_generator
from tvbextxircuits.nb_generator import NotebookGenerator, WidgetCodeGenerator

TEMP_DIR = "temp_test_dir"
nb_generator.NOTEBOOKS_DIR = TEMP_DIR


def test_notebook_generator():
    notebook_generator = NotebookGenerator()
    assert os.path.exists(nb_generator.NOTEBOOKS_DIR)
    assert notebook_generator.notebook is not None
    assert len(notebook_generator.notebook['cells']) == 0

    notebook_generator.add_code_cell("print('Hello world')")
    assert len(notebook_generator.notebook['cells']) == 1

    notebook_generator.add_markdown_cell("Hello world")
    assert len(notebook_generator.notebook['cells']) == 2

    assert len(os.listdir(nb_generator.NOTEBOOKS_DIR)) == 0
    notebook_path = notebook_generator.store('model', 'xircuits_id')
    assert os.path.exists(notebook_path)


def test_widget_code_generator():
    widget_code_generator = WidgetCodeGenerator()
    code = widget_code_generator.get_widget_code('Simulator', 'sim_id', 'xai_components/xai_tvb_simulator/simulator.py')
    assert code is None

    code = widget_code_generator.get_widget_code('ReducedWongWang', 'model_id',
                                                 'xai_components/xai_tvb_models/wong_wang.py')
    assert code is not None
    assert "models.ReducedWongWang()" in code
    assert "w.export_filename = 'model_model_id'" in code


def teardown_function():
    if os.path.exists(nb_generator.NOTEBOOKS_DIR):
        shutil.rmtree(nb_generator.NOTEBOOKS_DIR)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
from tvb.simulator.backend.templates import MakoUtilMix
from xai_components.base import xai_component, OutArg
from xai_components.base_tvb import TVBComponent


@xai_component(color='rgb(219, 63, 28)')
class NbMPRBackend(TVBComponent):
    nbMPRBackend: OutArg[MakoUtilMix]

    def __init__(self):
        self.done = False

        self.nbMPRBackend = OutArg(None)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.backend.nb_mpr import NbMPRBackend
        return NbMPRBackend

    def execute(self, ctx) -> None:

        self.nbMPRBackend.value = self.tvb_ht_class()



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#
import numpy as np
from tvb.basic.neotraits._attr import NArray, Final

from xai_components.base import OutArg, InArg

TYPE_STR = 'Type'
GID_STR = 'gid'


def should_set_attr(attr_value):
    """
    Check to see if an attribute should be set by the user or
    the default value should be kept
    """
    attr_name = attr_value.field_name
    if attr_name.lower() == 'gid':  # don't need to set the gid of an object in a Xircuits component
        return False
    if isinstance(attr_value, Final):  # if attr is Final, do not attempt to set it again
        return False

    return True


def set_defaults(component, tvb_datatype):
    # always set done attribute on False
    setattr(component, 'done', False)

    for attr in tvb_datatype.declarative_attrs:
        attr_value = getattr(tvb_datatype, attr)
        attr_default = attr_value.default
        if should_set_attr(attr_value):
            setattr(component, attr, InArg(attr_default))

    # also set the output arg
    out_arg_name = tvb_datatype.__name__
    out_arg_name = out_arg_name[0].lower() + out_arg_name[1:]
    setattr(component, out_arg_name, OutArg(None))


def set_values(component, tvb_object):
    cls = type(tvb_object)
    for attr in cls.declarative_attrs:
        attr_value = getattr(cls, attr)
        if should_set_attr(attr_value):
            xircuits_value = getattr(component, attr).value
            # if attribute in TVB is of type NArray, the list coming from Xircuits workflow must be converted to
            # numpy.array  to be able to set the value on the TVB object
            if isinstance(attr_value, NArray):
                # if default for attr is None and was not set by user do not attempt to set it
                if not xircuits_value:  # use any to also test arrays with multiple elements, otherwise error
                    continue

                dtype = attr_value.dtype.name  # needed for NArrays of int type
                xircuits_value = [xircuits_value]  # need to convert it to list first
                xircuits_value = np.array(object=xircuits_value, dtype=np.dtype(dtype))

                # after converting to np.ndarryay, the resulting shape might be (1,1),
                # which cannot be used in TVB computations, so a reshaping is needed
                if xircuits_value.shape == (1, 1):
                    xircuits_value = xircuits_value.reshape((1,))
                # xircuits_value = xircuits_value.squeeze()  # squeeze result so it can be used in TVB computations
            setattr(tvb_object, attr, xircuits_value)


def print_component_summary(traited_obj):
    print('== COMPONENT SUMMARY ==')
    summary = traited_obj.summary_info()
    print("{")

    # this if is needed because Connectivity doesn't add its type in its summary_info() method
    if TYPE_STR not in summary.keys():
        cls = type(traited_obj)
        print(f'"{TYPE_STR}": "{cls.__name__}",')

    for attr_name, attr in summary.items():
        print(f'"{attr_name}": "{attr}",')

    # this if is needed because Connectivity doesn't add its gid in its summary_info() method
    if GID_STR not in summary.keys():
        gid_value = repr(traited_obj.gid)
        print(f'"{GID_STR}": "{gid_value}"')
    print("}")



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class ZerlautAdaptationFirstOrder(ComponentWithWidget):
    g_L: InArg[float]
    E_L_e: InArg[float]
    E_L_i: InArg[float]
    C_m: InArg[float]
    b_e: InArg[float]
    a_e: InArg[float]
    b_i: InArg[float]
    a_i: InArg[float]
    tau_w_e: InArg[float]
    tau_w_i: InArg[float]
    E_e: InArg[float]
    E_i: InArg[float]
    Q_e: InArg[float]
    Q_i: InArg[float]
    tau_e: InArg[float]
    tau_i: InArg[float]
    N_tot: InArg[int]
    p_connect: InArg[float]
    g: InArg[float]
    K_ext_e: InArg[int]
    K_ext_i: InArg[int]
    T: InArg[float]
    P_e: InArg[float]
    P_i: InArg[float]
    external_input_ex_ex: InArg[float]
    external_input_ex_in: InArg[float]
    external_input_in_ex: InArg[float]
    external_input_in_in: InArg[float]
    variables_of_interest: InArg[list]

    zerlautAdaptationFirstOrder: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.zerlaut import ZerlautAdaptationFirstOrder
        return ZerlautAdaptationFirstOrder

    def execute(self, ctx) -> None:
        zerlautAdaptationFirstOrder = self.tvb_ht_class()

        set_values(self, zerlautAdaptationFirstOrder)
        self.zerlautAdaptationFirstOrder.value = zerlautAdaptationFirstOrder
        print_component_summary(self.zerlautAdaptationFirstOrder.value)


@xai_component(color='rgb(101, 179, 46)')
class ZerlautAdaptationSecondOrder(ComponentWithWidget):
    g_L: InArg[float]
    E_L_e: InArg[float]
    E_L_i: InArg[float]
    C_m: InArg[float]
    b_e: InArg[float]
    a_e: InArg[float]
    b_i: InArg[float]
    a_i: InArg[float]
    tau_w_e: InArg[float]
    tau_w_i: InArg[float]
    E_e: InArg[float]
    E_i: InArg[float]
    Q_e: InArg[float]
    Q_i: InArg[float]
    tau_e: InArg[float]
    tau_i: InArg[float]
    N_tot: InArg[int]
    p_connect: InArg[float]
    g: InArg[float]
    K_ext_e: InArg[int]
    K_ext_i: InArg[int]
    T: InArg[float]
    P_e: InArg[float]
    P_i: InArg[float]
    external_input_ex_ex: InArg[float]
    external_input_ex_in: InArg[float]
    external_input_in_ex: InArg[float]
    external_input_in_in: InArg[float]
    variables_of_interest: InArg[list]

    zerlautAdaptationSecondOrder: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.zerlaut import ZerlautAdaptationSecondOrder
        return ZerlautAdaptationSecondOrder

    def execute(self, ctx) -> None:
        zerlautAdaptationSecondOrder = self.tvb_ht_class()

        set_values(self, zerlautAdaptationSecondOrder)
        self.zerlautAdaptationSecondOrder.value = zerlautAdaptationSecondOrder
        print_component_summary(self.zerlautAdaptationSecondOrder.value)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.integrators import Integrator
from tvb.simulator.noise import Noise

from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import TVBComponent
from xai_components.utils import print_component_summary, set_defaults, set_values


@xai_component(color='rgb(253, 225, 0)')
class HeunDeterministic(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]

    heunDeterministic: OutArg[Integrator]


    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import HeunDeterministic
        return HeunDeterministic

    def execute(self, ctx) -> None:
        heunDeterministic = self.tvb_ht_class()

        set_values(self, heunDeterministic)
        self.heunDeterministic.value = heunDeterministic
        print_component_summary(self.heunDeterministic.value)


@xai_component(color='rgb(253, 225, 0)')
class HeunStochastic(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]
    noise: InArg[Noise]

    heunStochastic: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import HeunStochastic
        return HeunStochastic

    def execute(self, ctx) -> None:
        heunStochastic = self.tvb_ht_class()

        set_values(self, heunStochastic)
        self.heunStochastic.value = heunStochastic
        print_component_summary(self.heunStochastic.value)


@xai_component(color='rgb(253, 225, 0)')
class EulerDeterministic(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]

    eulerDeterministic: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import EulerDeterministic
        return EulerDeterministic

    def execute(self, ctx) -> None:
        eulerDeterministic = self.tvb_ht_class()

        set_values(self, eulerDeterministic)
        self.eulerDeterministic.value = eulerDeterministic
        print_component_summary(self.eulerDeterministic.value)


@xai_component(color='rgb(253, 225, 0)')
class EulerStochastic(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]
    noise: InArg[Noise]

    eulerStochastic: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import EulerStochastic
        return EulerStochastic

    def execute(self, ctx) -> None:
        eulerStochastic = self.tvb_ht_class()

        set_values(self, eulerStochastic)
        self.eulerStochastic.value = eulerStochastic
        print_component_summary(self.eulerStochastic.value)


@xai_component(color='rgb(253, 225, 0)')
class RungeKutta4thOrderDeterministic(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]

    rungeKutta4thOrderDeterministic: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import RungeKutta4thOrderDeterministic
        return RungeKutta4thOrderDeterministic


    def execute(self, ctx) -> None:
        rungeKutta4thOrderDeterministic = self.tvb_ht_class()

        set_values(self, rungeKutta4thOrderDeterministic)
        self.rungeKutta4thOrderDeterministic.value = rungeKutta4thOrderDeterministic
        print_component_summary(self.rungeKutta4thOrderDeterministic.value)


@xai_component(color='rgb(253, 225, 0)')
class Identity(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]

    identity: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import Identity
        return Identity

    def execute(self, ctx) -> None:
        identity = self.tvb_ht_class()

        set_values(self, identity)
        self.identity.value = identity
        print_component_summary(self.identity.value)


@xai_component(color='rgb(253, 225, 0)')
class IdentityStochastic(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]
    noise: InArg[Noise]

    identityStochastic: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import IdentityStochastic
        return IdentityStochastic

    def execute(self, ctx) -> None:
        identityStochastic = self.tvb_ht_class()

        set_values(self, identityStochastic)
        self.identityStochastic.value = identityStochastic
        print_component_summary(self.identityStochastic.value)


@xai_component(color='rgb(253, 225, 0)')
class VODE(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]

    vODE: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import VODE
        return VODE

    def execute(self, ctx) -> None:
        vODE = self.tvb_ht_class()

        set_values(self, vODE)
        self.vODE.value = vODE
        print_component_summary(self.vODE.value)


@xai_component(color='rgb(253, 225, 0)')
class VODEStochastic(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]
    noise: InArg[Noise]

    vODEStochastic: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import VODEStochastic
        return VODEStochastic

    def execute(self, ctx) -> None:
        vODEStochastic = self.tvb_ht_class()

        set_values(self, vODEStochastic)
        self.vODEStochastic.value = vODEStochastic
        print_component_summary(self.vODEStochastic.value)


@xai_component(color='rgb(253, 225, 0)')
class Dopri5(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]

    dopri5: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import Dopri5
        return Dopri5

    def execute(self, ctx) -> None:
        dopri5 = self.tvb_ht_class()

        set_values(self, dopri5)
        self.dopri5.value = dopri5
        print_component_summary(self.dopri5.value)


@xai_component(color='rgb(253, 225, 0)')
class Dopri5Stochastic(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]
    noise: InArg[Noise]

    dopri5Stochastic: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import Dopri5Stochastic
        return Dopri5Stochastic

    def execute(self, ctx) -> None:
        dopri5Stochastic = self.tvb_ht_class()

        set_values(self, dopri5Stochastic)
        self.dopri5Stochastic.value = dopri5Stochastic
        print_component_summary(self.dopri5Stochastic.value)


@xai_component(color='rgb(253, 225, 0)')
class Dop853(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]

    dop853: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import Dop853
        return Dop853

    def execute(self, ctx) -> None:
        dop853 = self.tvb_ht_class()

        set_values(self, dop853)
        self.dop853.value = dop853
        print_component_summary(self.dop853.value)


@xai_component(color='rgb(253, 225, 0)')
class Dop853Stochastic(TVBComponent):
    dt: InArg[float]
    bounded_state_variable_indices: InArg[list]
    state_variable_boundaries: InArg[list]
    clamped_state_variable_indices: InArg[list]
    clamped_state_variable_values: InArg[list]
    noise: InArg[Noise]

    dop853Stochastic: OutArg[Integrator]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.integrators import Dop853Stochastic
        return Dop853Stochastic

    def execute(self, ctx) -> None:
        dop853Stochastic = self.tvb_ht_class()

        set_values(self, dop853Stochastic)
        self.dop853Stochastic.value = dop853Stochastic
        print_component_summary(self.dop853Stochastic.value)


from urllib import request
from git import Repo
from git.remote import RemoteProgress

class Progress(RemoteProgress):
    def update(self, *args):
        print(self._cur_line, end='\r', flush=True)

def get_submodule_config(user_query):
    
    import configparser
    config = configparser.ConfigParser()
    
    config.read('.xircuits/.gitmodules')
    
    submodule_keys = [submodule for submodule in config.sections() if user_query in submodule]
    if len(submodule_keys) == 0:
        print(user_query + " component library submodule not found.")
        return
        
    if len(submodule_keys) > 1:
        print("Multiple '" + user_query + "' found. Returning first instance.")

    submodule_key = submodule_keys.pop(0)
    
    submodule_path = config[submodule_key]["path"]
    submodule_url = config[submodule_key]["url"]
    
    return submodule_path, submodule_url


def request_submodule_library(component_library_query):

    module_url = "https://raw.githubusercontent.com/XpressAI/xircuits/master/.gitmodules"
    request.urlretrieve(module_url, ".xircuits/.gitmodules")
    
    # ensure syntax is as xai_components/xai_library_name
    if "xai" not in component_library_query:
        component_library_query = "xai_" + component_library_query

    if "xai_components" not in component_library_query:
        component_library_query = "xai_components/" + component_library_query
    
    submodule_path, submodule_url = get_submodule_config(component_library_query)
    
    print("Cloning " + submodule_path + " from " + submodule_url)
    Repo.clone_from(submodule_url, submodule_path, progress=Progress())



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class WilsonCowan(ComponentWithWidget):
    c_ee: InArg[float]
    c_ei: InArg[float]
    c_ie: InArg[float]
    c_ii: InArg[float]
    tau_e: InArg[float]
    tau_i: InArg[float]
    a_e: InArg[float]
    b_e: InArg[float]
    c_e: InArg[float]
    theta_e: InArg[float]
    a_i: InArg[float]
    b_i: InArg[float]
    theta_i: InArg[float]
    c_i: InArg[float]
    r_e: InArg[float]
    r_i: InArg[float]
    k_e: InArg[float]
    k_i: InArg[float]
    P: InArg[float]
    Q: InArg[float]
    alpha_e: InArg[float]
    alpha_i: InArg[float]
    shift_sigmoid: InArg[bool]
    variables_of_interest: InArg[list]

    wilsonCowan: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.wilson_cowan import WilsonCowan
        return WilsonCowan

    def execute(self, ctx) -> None:
        wilsonCowan = self.tvb_ht_class()

        set_values(self, wilsonCowan)
        self.wilsonCowan.value = wilsonCowan
        print_component_summary(self.wilsonCowan.value)



import os
import re

FILE_ARGUMENT = 'file_path'    # name of InArg for HPC file upload


def get_file_path_from_line(line):
    """
    Get the absolute path of a file provided as input argument to a Xircuits component
    :param line: line where the file_input param. of a component gets a value (a path from disk)
    :return: absolute path to the file
    """
    file_path = line.split('"""')[1]    # the relative file path is always between these this sequence of characters
    abs_file_path = os.path.abspath(file_path)  # absolute file path
    return abs_file_path


def get_file_name_from_line(line):
    """
    Get just the file name of a file provided as input argument to a Xircuits component
    :param line: line where the file_input param. of a component gets a value (a relative path from disk)
    :return: the file name of the file
    """
    file_path = line.split('"""')[1]    # the relative file path is always between these this sequence of characters
    file_name = os.path.basename(file_path)
    return file_name


def get_files_to_upload(xircuits_file_path):
    """
    Retrieve the list of files that need to be uploaded to the HPC.
    Also modify the compiled xircuits file, such that for the components using the uploaded files, only the file
    names are specified, not their path (relative or absolute).
    :param xircuits_file_path: path to the compiled xircuits workflow file
    :return: list containing the files that need to be uploaded to the HPC
    """
    print("Gathering input files...", flush=True)
    files_to_upload = []  # will contain the paths of all files that need to be uploaded
    new_file = []   # will contain the xircuits .py file, but keeping only the file names, not their whole relative path
    with open(xircuits_file_path) as f:
        lines = f.readlines()

    # if FILE_ARGUMENT is nowhere in the compiled .py file, don't search for it in every line
    # TODO: make this nicer
    if any(FILE_ARGUMENT in line for line in lines):
        for line in lines:
            if FILE_ARGUMENT in line:
                # store the path of the file that needs to be uploaded
                file_to_upload = get_file_path_from_line(line)
                files_to_upload.append(file_to_upload)

                # replace in the compiled .py file the relative path with the file name for `file_path` arguments
                new_line = line
                input_file_name = get_file_name_from_line(line)
                new_line = re.sub('""".*?"""', '"""' + input_file_name + '"""', new_line)
                new_file.append(new_line)
            else:
                new_file.append(line)

        # rewrite the compiled .py file:
        with open(xircuits_file_path, 'w') as f:
            f.writelines(new_file)

    return files_to_upload




# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


# TODO: uncomment once EpileptorT is fixed in tvb-library
# @xai_component(color='rgb(101, 179, 46)')
# class EpileptorT(ComponentWithWidget):
#     a: InArg[float]
#     b: InArg[float]
#     c: InArg[float]
#     d: InArg[float]
#     r: InArg[float]
#     s: InArg[float]
#     x0: InArg[float]
#     Iext: InArg[float]
#     slope: InArg[float]
#     Iext2: InArg[float]
#     tau: InArg[float]
#     aa: InArg[float]
#     bb: InArg[float]
#     Kvf: InArg[float]
#     Kf: InArg[float]
#     Ks: InArg[float]
#     tt: InArg[float]
#     modification: InArg[int]
#     variables_of_interest: InArg[list]
#
#     epileptorT: OutArg[Model]
#
#     @property
#     def tvb_ht_class(self):
#         from tvb.simulator.models.epileptorT import EpileptorT
#         return EpileptorT
#
#     def execute(self, ctx) -> None:
#         epileptorT = self.tvb_ht_class()
#
#         set_values(self, epileptorT)
#         self.epileptorT.value = epileptorT
#         print_component_summary(self.epileptorT.value)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class ReducedWongWang(ComponentWithWidget):
    a: InArg[float]
    b: InArg[float]
    d: InArg[float]
    gamma: InArg[float]
    tau_s: InArg[float]
    w: InArg[float]
    J_N: InArg[float]
    I_o: InArg[float]
    sigma_noise: InArg[float]
    variables_of_interest: InArg[list]

    reducedWongWang: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.wong_wang import ReducedWongWang
        return ReducedWongWang

    def execute(self, ctx) -> None:
        reducedWongWang = self.tvb_ht_class()

        set_values(self, reducedWongWang)
        self.reducedWongWang.value = reducedWongWang
        print_component_summary(self.reducedWongWang.value)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

import importlib
import json
import os
import nbformat
from tvb.simulator.integrators import HeunDeterministic
from tvb.simulator.models.oscillator import Generic2dOscillator

from xai_components.base_tvb import ComponentWithWidget

NOTEBOOKS_DIR = 'TVB_generated_notebooks'
MODEL_CONFIG_FILE_PREFIX = 'model'


class ModelConfigLoader(object):

    def __init__(self, xircuits_id):
        self.model_configs_folder = os.path.join(NOTEBOOKS_DIR, xircuits_id)

    def load_configs(self):
        all_model_config_files = self._find_model_config_files()

        if all_model_config_files is False:
            return False

        json_result = dict()
        for filename in all_model_config_files:
            json_entry = self._read_model_config_file(filename)
            json_result.update(json_entry)
            self._remove_file(filename)

        return json_result

    def _find_model_config_files(self):
        if not os.path.exists(self.model_configs_folder):
            return False

        all_files = os.listdir(self.model_configs_folder)
        all_model_config_files = [file for file in all_files if file.startswith(MODEL_CONFIG_FILE_PREFIX)]

        if len(all_model_config_files) == 0:
            return False

        return all_model_config_files

    def _read_model_config_file(self, filename):
        # TODO: better processing of filename
        model_id = filename[len(MODEL_CONFIG_FILE_PREFIX) + 1:].split('.')[0]

        model_name = 'model'
        model_params_json = dict()
        with open(os.path.join(self.model_configs_folder, filename)) as f:
            model_config_json = json.load(f)
            for param_name, param_val in list(model_config_json.values())[0].items():
                if param_name == 'model':
                    model_name = param_val
                    continue
                param_entry = {
                    param_name: {'name': param_name, 'value': param_val[0], 'type': type(param_val[0]).__name__}}
                model_params_json.update(param_entry)

        return {
            model_name: {
                "id": model_id,
                "params": model_params_json
            }
        }

    def _remove_file(self, filename):
        os.remove(os.path.join(self.model_configs_folder, filename))
        if len(os.listdir(self.model_configs_folder)) == 0:
            os.rmdir(self.model_configs_folder)


class NotebookGenerator(object):

    def __init__(self):
        if not os.path.exists(NOTEBOOKS_DIR):
            os.mkdir(NOTEBOOKS_DIR)

        self.notebook = nbformat.v4.new_notebook()

    def add_code_cell(self, code):
        self._add_cell(nbformat.v4.new_code_cell(code, metadata={'editable': False, 'deletable': False}))

    def add_markdown_cell(self, text):
        self._add_cell(nbformat.v4.new_markdown_cell(text))

    def _add_cell(self, cell):
        self.notebook['cells'].append(cell)

    def store(self, component, xircuits_id):
        file_name = f'{component}_widget.ipynb'

        notebook_dir = os.path.join(NOTEBOOKS_DIR, xircuits_id)
        if not os.path.exists(notebook_dir):
            os.mkdir(notebook_dir)

        path = os.path.join(notebook_dir, file_name)

        with open(path, 'w') as f:
            nbformat.write(self.notebook, f)

        return os.path.join(notebook_dir, file_name)


class WidgetCodeGenerator(object):

    @staticmethod
    def phase_plane(model=Generic2dOscillator, integrator=HeunDeterministic, export_filename=None):
        code = "%matplotlib widget\n" \
               "\n" \
               "from tvb.simulator.lab import models, integrators\n" \
               "from tvbwidgets.api import PhasePlaneWidget\n" \
               "from IPython.core.display_functions import display\n" \
               "\n" \
               "w = PhasePlaneWidget(model=models.{0}(), integrator=integrators.{1}());\n" \
               "w.export_filename = '{2}' \n" \
               "w.disable_model_dropdown = True \n" \
               "w.disable_export_dropdown = True \n" \
               "display(w.get_widget());"
        return code.format(model.__name__, integrator.__name__, export_filename)

    @staticmethod
    def get_widget_code(component_name, component_id, component_path):
        component_class = determine_component_class(component_name, component_path)

        if issubclass(component_class, ComponentWithWidget):
            return WidgetCodeGenerator.phase_plane(component_class().tvb_ht_class,
                                                   export_filename=f"{MODEL_CONFIG_FILE_PREFIX}_{component_id}")

        else:
            return None


def determine_component_class(component_name, component_path):
    component_module = importlib.import_module(component_path.replace('/', '.')[:-3])
    component_class = getattr(component_module, component_name)
    return component_class



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class JansenRit(ComponentWithWidget):
    A: InArg[float]
    B: InArg[float]
    a: InArg[float]
    b: InArg[float]
    v0: InArg[float]
    nu_max: InArg[float]
    r: InArg[float]
    J: InArg[float]
    a_1: InArg[float]
    a_2: InArg[float]
    a_3: InArg[float]
    a_4: InArg[float]
    p_min: InArg[float]
    p_max: InArg[float]
    mu: InArg[float]
    variables_of_interest: InArg[list]

    jansenRit: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.jansen_rit import JansenRit
        return JansenRit

    def execute(self, ctx) -> None:
        jansenRit = self.tvb_ht_class()

        set_values(self, jansenRit)
        self.jansenRit.value = jansenRit
        print_component_summary(self.jansenRit.value)


@xai_component(color='rgb(101, 179, 46)')
class ZetterbergJansen(ComponentWithWidget):
    He: InArg[float]
    Hi: InArg[float]
    ke: InArg[float]
    ki: InArg[float]
    e0: InArg[float]
    rho_2: InArg[float]
    rho_1: InArg[float]
    gamma_1: InArg[float]
    gamma_2: InArg[float]
    gamma_3: InArg[float]
    gamma_4: InArg[float]
    gamma_5: InArg[float]
    gamma_1T: InArg[float]
    gamma_2T: InArg[float]
    gamma_3T: InArg[float]
    P: InArg[float]
    U: InArg[float]
    Q: InArg[float]
    variables_of_interest: InArg[list]

    zetterbergJansen: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.jansen_rit import ZetterbergJansen
        return ZetterbergJansen

    def execute(self, ctx) -> None:
        zetterbergJansen = self.tvb_ht_class()

        set_values(self, zetterbergJansen)
        self.zetterbergJansen.value = zetterbergJansen
        print_component_summary(self.zetterbergJansen.value)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#
from tvb.basic.neotraits.api import EnumAttr
from tvb.datatypes.equations import HRFKernelEquation
from tvb.datatypes.projections import ProjectionSurfaceEEG, ProjectionSurfaceMEG, ProjectionSurfaceSEEG
from tvb.datatypes.region_mapping import RegionMapping
from tvb.datatypes.sensors import SensorsEEG, SensorsInternal, SensorsMEG
from tvb.simulator.noise import Noise

from xai_components.base import xai_component, InArg, OutArg, InCompArg
from xai_components.base_tvb import TVBComponent
from xai_components.utils import print_component_summary, set_defaults, set_values


@xai_component(color='rgb(247, 158, 27)')
class Raw(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]

    raw: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import Raw
        return Raw

    def execute(self, ctx) -> None:
        raw = self.tvb_ht_class()
        set_values(self, raw)

        monitors_list = self.previous_monitors.value
        monitors_list.append(raw)
        self.raw.value = monitors_list
        print_component_summary(self.raw.value[-1]) # -1 to print the latest added monitor, i.e. the current monitor


@xai_component(color='rgb(247, 158, 27)')
class RawVoi(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]

    rawVoi: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import RawVoi
        return RawVoi

    def execute(self, ctx) -> None:
        rawVoi = self.tvb_ht_class()
        set_values(self, rawVoi)

        monitors_list = self.previous_monitors.value
        monitors_list.append(rawVoi)
        self.rawVoi.value = monitors_list
        print_component_summary(self.rawVoi.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class SubSample(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]

    subSample: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import SubSample
        return SubSample

    def execute(self, ctx) -> None:
        subSample = self.tvb_ht_class()
        set_values(self, subSample)

        monitors_list = self.previous_monitors.value
        monitors_list.append(subSample)
        self.subSample.value = monitors_list
        print_component_summary(self.subSample.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class SpatialAverage(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]
    spatial_mask: InArg[list]
    default_mask: InArg[EnumAttr]

    spatialAverage: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import SpatialAverage
        return SpatialAverage

    def execute(self, ctx) -> None:
        spatialAverage = self.tvb_ht_class()
        set_values(self, spatialAverage)

        monitors_list = self.previous_monitors.value
        monitors_list.append(spatialAverage)
        self.spatialAverage.value = monitors_list
        print_component_summary(self.spatialAverage.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class GlobalAverage(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]

    globalAverage: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import GlobalAverage
        return GlobalAverage

    def execute(self, ctx) -> None:
        globalAverage = self.tvb_ht_class()
        set_values(self, globalAverage)

        monitors_list = self.previous_monitors.value
        monitors_list.append(globalAverage)
        self.globalAverage.value = monitors_list
        print_component_summary(self.globalAverage.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class TemporalAverage(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]

    temporalAverage: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import TemporalAverage
        return TemporalAverage

    def execute(self, ctx) -> None:
        temporalAverage = self.tvb_ht_class()
        set_values(self, temporalAverage)

        monitors_list = self.previous_monitors.value
        monitors_list.append(temporalAverage)
        self.temporalAverage.value = monitors_list
        print_component_summary(self.temporalAverage.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class AfferentCoupling(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]

    afferentCoupling: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import AfferentCoupling
        return AfferentCoupling

    def execute(self, ctx) -> None:
        afferentCoupling = self.tvb_ht_class()
        set_values(self, afferentCoupling)

        monitors_list = self.previous_monitors.value
        monitors_list.append(afferentCoupling)
        self.afferentCoupling.value = monitors_list
        print_component_summary(self.afferentCoupling.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class AfferentCouplingTemporalAverage(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]

    afferentCouplingTemporalAverage: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import AfferentCouplingTemporalAverage
        return AfferentCouplingTemporalAverage

    def execute(self, ctx) -> None:
        afferentCouplingTemporalAverage = self.tvb_ht_class()
        set_values(self, afferentCouplingTemporalAverage)

        monitors_list = self.previous_monitors.value
        monitors_list.append(afferentCouplingTemporalAverage)
        self.afferentCouplingTemporalAverage.value = monitors_list
        print_component_summary(self.afferentCouplingTemporalAverage.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class EEG(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]
    region_mapping: InArg[RegionMapping]
    obsnoise: InArg[Noise]
    projection: InCompArg[ProjectionSurfaceEEG]
    reference: InArg[str]
    sensors: InCompArg[SensorsEEG]
    sigma: InArg[float]

    eEG: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import EEG
        return EEG

    def execute(self, ctx) -> None:
        eEG = self.tvb_ht_class()
        set_values(self, eEG)

        monitors_list = self.previous_monitors.value
        monitors_list.append(eEG)
        self.eEG.value = monitors_list
        print_component_summary(self.eEG.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class MEG(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]
    region_mapping: InArg[RegionMapping]
    obsnoise: InArg[Noise]
    projection: InCompArg[ProjectionSurfaceMEG]
    sensors: InCompArg[SensorsMEG]

    mEG: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import MEG
        return MEG

    def execute(self, ctx) -> None:
        mEG = self.tvb_ht_class()
        set_values(self, mEG)

        monitors_list = self.previous_monitors.value
        monitors_list.append(mEG)
        self.mEG.value = monitors_list
        print_component_summary(self.mEG.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class iEEG(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]
    region_mapping: InArg[RegionMapping]
    obsnoise: InArg[Noise]
    projection: InCompArg[ProjectionSurfaceSEEG]
    sigma: InArg[float]
    sensors: InCompArg[SensorsInternal]

    iEEG: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import iEEG
        return iEEG

    def execute(self, ctx) -> None:
        iEEG = self.tvb_ht_class()
        set_values(self, iEEG)

        monitors_list = self.previous_monitors.value
        monitors_list.append(iEEG)
        self.iEEG.value = monitors_list
        print_component_summary(self.iEEG.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class Bold(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]
    hrf_kernel: InArg[HRFKernelEquation]
    hrf_length: InArg[float]

    bold: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import Bold
        return Bold

    def execute(self, ctx) -> None:
        bold = self.tvb_ht_class()
        set_values(self, bold)

        monitors_list = self.previous_monitors.value
        monitors_list.append(bold)
        self.bold.value = monitors_list
        print_component_summary(self.bold.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class BoldRegionROI(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]
    hrf_kernel: InArg[HRFKernelEquation]
    hrf_length: InArg[float]

    boldRegionROI: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import BoldRegionROI
        return BoldRegionROI

    def execute(self, ctx) -> None:
        boldRegionROI = self.tvb_ht_class()
        set_values(self, boldRegionROI)

        monitors_list = self.previous_monitors.value
        monitors_list.append(boldRegionROI)
        self.boldRegionROI.value = monitors_list
        print_component_summary(self.boldRegionROI.value[-1])


@xai_component(color='rgb(247, 158, 27)')
class ProgressLogger(TVBComponent):
    previous_monitors: InArg[list]
    period: InArg[float]
    variables_of_interest: InArg[list]

    progressLogger: OutArg[list]

    def __init__(self):
        self.previous_monitors = InArg([])
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.monitors import ProgressLogger
        return ProgressLogger

    def execute(self, ctx) -> None:
        progressLogger = self.tvb_ht_class()
        set_values(self, progressLogger)

        monitors_list = self.previous_monitors.value
        monitors_list.append(progressLogger)
        self.progressLogger.value = monitors_list
        print_component_summary(self.progressLogger.value[-1])


from tvb.datatypes.sensors import Sensors

from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import TVBComponent
from xai_components.utils import set_values, print_component_summary, set_defaults


@xai_component(color='rgb(0, 116, 149)')
class SensorsEEG(TVBComponent):
    file_path: InArg[str]
    has_orientation: InArg[bool]
    orientations: InArg[list]
    usable: InArg[list]

    sensorsEEG: OutArg[Sensors]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.sensors import SensorsEEG
        return SensorsEEG

    def __init__(self):
        self.file_path = InArg(None)
        set_defaults(self, self.tvb_ht_class)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'eeg_brainstorm_65.txt'  # default from tvb_data
        sensorsEEG = self.tvb_ht_class.from_file(source_file=file_path)
        set_values(self, sensorsEEG)
        sensorsEEG.configure()

        self.sensorsEEG.value = sensorsEEG
        print_component_summary(self.sensorsEEG.value)


@xai_component(color='rgb(0, 116, 149)')
class SensorsMEG(TVBComponent):
    file_path: InArg[str]
    usable: InArg[list]

    sensorsMEG: OutArg[Sensors]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.sensors import SensorsMEG
        return SensorsMEG

    def __init__(self):
        self.file_path = InArg(None)
        set_defaults(self, self.tvb_ht_class)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'meg_151.txt.bz2'  # default from tvb_data
        sensorsMEG = self.tvb_ht_class.from_file(source_file=file_path)
        set_values(self, sensorsMEG)
        sensorsMEG.configure()

        self.sensorsMEG.value = sensorsMEG
        print_component_summary(self.sensorsMEG.value)


@xai_component(color='rgb(0, 116, 149)')
class SensorsInternal(TVBComponent):
    file_path: InArg[str]
    has_orientation: InArg[bool]
    orientations: InArg[list]
    usable: InArg[list]

    sensorsInternal: OutArg[Sensors]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.sensors import SensorsInternal
        return SensorsInternal

    def __init__(self):
        self.file_path = InArg(None)
        set_defaults(self, self.tvb_ht_class)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'seeg_39.txt.bz2'  # default from tvb_data
        sensorsInternal = self.tvb_ht_class.from_file(source_file=file_path)
        set_values(self, sensorsInternal)
        sensorsInternal.configure()

        self.sensorsInternal.value = sensorsInternal
        print_component_summary(self.sensorsInternal.value)


# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class EpileptorCodim3(ComponentWithWidget):
    mu1_start: InArg[float]
    mu2_start: InArg[float]
    nu_start: InArg[float]
    mu1_stop: InArg[float]
    mu2_stop: InArg[float]
    nu_stop: InArg[float]
    b: InArg[float]
    R: InArg[float]
    c: InArg[float]
    dstar: InArg[float]
    Ks: InArg[float]
    N: InArg[float]
    modification: InArg[bool]
    variables_of_interest: InArg[list]

    epileptorCodim3: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.epileptorcodim3 import EpileptorCodim3
        return EpileptorCodim3

    def execute(self, ctx) -> None:
        epileptorCodim3 = self.tvb_ht_class()

        set_values(self, epileptorCodim3)
        self.epileptorCodim3.value = epileptorCodim3
        print_component_summary(self.epileptorCodim3.value)


@xai_component(color='rgb(101, 179, 46)')
class EpileptorCodim3SlowMod(ComponentWithWidget):
    mu1_Ain: InArg[float]
    mu2_Ain: InArg[float]
    nu_Ain: InArg[float]
    mu1_Bin: InArg[float]
    mu2_Bin: InArg[float]
    nu_Bin: InArg[float]
    mu1_Aend: InArg[float]
    mu2_Aend: InArg[float]
    nu_Aend: InArg[float]
    mu1_Bend: InArg[float]
    mu2_Bend: InArg[float]
    nu_Bend: InArg[float]
    b: InArg[float]
    R: InArg[float]
    c: InArg[float]
    cA: InArg[float]
    cB: InArg[float]
    dstar: InArg[float]
    Ks: InArg[float]
    N: InArg[float]
    modification: InArg[bool]
    variables_of_interest: InArg[list]

    epileptorCodim3SlowMod: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.epileptorcodim3 import EpileptorCodim3SlowMod
        return EpileptorCodim3SlowMod

    def execute(self, ctx) -> None:
        epileptorCodim3SlowMod = self.tvb_ht_class()

        set_values(self, epileptorCodim3SlowMod)
        self.epileptorCodim3SlowMod.value = epileptorCodim3SlowMod
        print_component_summary(self.epileptorCodim3SlowMod.value)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#
from numpy.random import RandomState
from tvb.simulator.noise import Noise

from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import TVBComponent
from xai_components.utils import print_component_summary, set_defaults, set_values


@xai_component(color='rgb(253, 225, 0)')
class Additive(TVBComponent):
    ntau: InArg[float]
    noise_seed: InArg[int]
    random_stream: InArg[RandomState]
    nsig: InArg[float]

    additive: OutArg[Noise]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.noise import Additive
        return Additive

    def execute(self, ctx) -> None:
        additive = self.tvb_ht_class()

        set_values(self, additive)
        self.additive.value = additive
        print_component_summary(self.additive.value)


@xai_component(color='rgb(253, 225, 0)')
class Multiplicative(TVBComponent):
    from tvb.datatypes.equations import TemporalApplicableEquation
    ntau: InArg[float]
    noise_seed: InArg[int]
    random_stream: InArg[RandomState]
    nsig: InArg[float]
    b: InArg[TemporalApplicableEquation]

    multiplicative: OutArg[Noise]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.noise import Multiplicative
        return Multiplicative

    def execute(self, ctx) -> None:
        multiplicative = self.tvb_ht_class()

        set_values(self, multiplicative)
        self.multiplicative.value = multiplicative
        print_component_summary(self.multiplicative.value)




# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class Generic2dOscillator(ComponentWithWidget):
    tau: InArg[float]
    I: InArg[float]
    a: InArg[float]
    b: InArg[float]
    c: InArg[float]
    d: InArg[float]
    e: InArg[float]
    f: InArg[float]
    g: InArg[float]
    alpha: InArg[float]
    beta: InArg[float]
    gamma: InArg[float]
    variables_of_interest: InArg[list]

    generic2dOscillator: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.oscillator import Generic2dOscillator
        return Generic2dOscillator

    def execute(self, ctx) -> None:
        generic2dOscillator = self.tvb_ht_class()

        set_values(self, generic2dOscillator)
        self.generic2dOscillator.value = generic2dOscillator
        print_component_summary(self.generic2dOscillator.value)


@xai_component(color='rgb(101, 179, 46)')
class Kuramoto(ComponentWithWidget):
    omega: InArg[float]
    variables_of_interest: InArg[list]

    kuramoto: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.oscillator import Kuramoto
        return Kuramoto

    def execute(self, ctx) -> None:
        kuramoto = self.tvb_ht_class()

        set_values(self, kuramoto)
        self.kuramoto.value = kuramoto
        print_component_summary(self.kuramoto.value)


@xai_component(color='rgb(101, 179, 46)')
class SupHopf(ComponentWithWidget):
    a: InArg[float]
    omega: InArg[float]
    variables_of_interest: InArg[list]

    supHopf: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.oscillator import SupHopf
        return SupHopf

    def execute(self, ctx) -> None:
        supHopf = self.tvb_ht_class()

        set_values(self, supHopf)
        self.supHopf.value = supHopf
        print_component_summary(self.supHopf.value)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class Hopfield(ComponentWithWidget):
    taux: InArg[float]
    tauT: InArg[float]
    dynamic: InArg[int]
    variables_of_interest: InArg[list]

    hopfield: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.hopfield import Hopfield
        return Hopfield

    def execute(self, ctx) -> None:
        hopfield = self.tvb_ht_class()

        set_values(self, hopfield)
        self.hopfield.value = hopfield
        print_component_summary(self.hopfield.value)



from tvb.basic.neotraits.api import HasTraits
from rst2html import rst2html
import json


def doc_to_html(doc):
    html, _ = rst2html(doc, report_level=5)
    github_imgs_url = r'https://raw.githubusercontent.com/the-virtual-brain/tvb-root/master/tvb_documentation/sim_doc/'
    return html.replace(r'<object data="', r'<img width="500" height="500" src="' + github_imgs_url)\
        .replace(r'</object>', r'</img>')


def get_class(kls):
    parts = kls.split('.')
    module = ".".join(parts[:-1])
    m = __import__(module)
    for comp in parts[1:]:
        m = getattr(m, comp)
    return m


def object_parse(object):

    tvb_object = object.tvb_ht_class


    output_dict = {
        "description": doc_to_html(tvb_object.__doc__) if tvb_object.__doc__ else "",
        "arguments": {}
    }

    try:
        if issubclass(tvb_object, HasTraits):
            for attr_name in tvb_object.declarative_attrs:
                doc = getattr(tvb_object, attr_name).doc
                if doc != "":
                    output_dict['arguments'][attr_name] = doc_to_html(doc) if doc else ""
    except TypeError:
        print(str(tvb_object) + "     " + str(type(tvb_object)))

    return output_dict


def save_json_description(class_path, output_path):
    class_obj = get_class(class_path)()
    description_dictionary = object_parse(class_obj)
    json_object = json.dumps(description_dictionary, indent=4)

    with open(output_path, "w+") as outfile:
        outfile.write(json_object)





# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.coupling import Coupling
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import TVBComponent
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(0, 102, 178)')
class LinearCoupling(TVBComponent):
    a: InArg[float]
    b: InArg[float]

    linear: OutArg[Coupling]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.coupling import Linear
        return Linear

    def execute(self, ctx) -> None:
        linear = self.tvb_ht_class()

        set_values(self, linear)
        self.linear.value = linear
        print_component_summary(self.linear.value)


@xai_component(color='rgb(0, 102, 178)')
class Scaling(TVBComponent):
    a: InArg[float]

    scaling: OutArg[Coupling]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.coupling import Scaling
        return Scaling

    def execute(self, ctx) -> None:
        scaling = self.tvb_ht_class()

        set_values(self, scaling)
        self.scaling.value = scaling
        print_component_summary(self.scaling.value)


@xai_component(color='rgb(0, 102, 178)')
class HyperbolicTangent(TVBComponent):
    a: InArg[float]
    b: InArg[float]
    midpoint: InArg[float]
    sigma: InArg[float]

    hyperbolicTangent: OutArg[Coupling]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.coupling import HyperbolicTangent
        return HyperbolicTangent

    def execute(self, ctx) -> None:
        hyperbolicTangent = self.tvb_ht_class()

        set_values(self, hyperbolicTangent)
        self.hyperbolicTangent.value = hyperbolicTangent
        print_component_summary(self.hyperbolicTangent.value)


@xai_component(color='rgb(0, 102, 178)')
class Sigmoidal(TVBComponent):
    cmin: InArg[float]
    cmax: InArg[float]
    midpoint: InArg[float]
    a: InArg[float]
    sigma: InArg[float]

    sigmoidal: OutArg[Coupling]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.coupling import Sigmoidal
        return Sigmoidal

    def execute(self, ctx) -> None:
        sigmoidal = self.tvb_ht_class()

        set_values(self, sigmoidal)
        self.sigmoidal.value = sigmoidal
        print_component_summary(self.sigmoidal.value)


@xai_component(color='rgb(0, 102, 178)')
class SigmoidalJansenRit(TVBComponent):
    cmin: InArg[float]
    cmax: InArg[float]
    midpoint: InArg[float]
    r: InArg[float]
    a: InArg[float]

    sigmoidalJansenRit: OutArg[Coupling]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.coupling import SigmoidalJansenRit
        return SigmoidalJansenRit

    def execute(self, ctx) -> None:
        sigmoidalJansenRit = self.tvb_ht_class()

        set_values(self, sigmoidalJansenRit)
        self.sigmoidalJansenRit.value = sigmoidalJansenRit
        print_component_summary(self.sigmoidalJansenRit.value)


@xai_component(color='rgb(0, 102, 178)')
class PreSigmoidal(TVBComponent):
    H: InArg[float]
    Q: InArg[float]
    G: InArg[float]
    P: InArg[float]
    theta: InArg[float]
    dynamic: InArg[bool]
    globalT: InArg[bool]

    preSigmoidal: OutArg[Coupling]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.coupling import PreSigmoidal
        return PreSigmoidal

    def execute(self, ctx) -> None:
        preSigmoidal = self.tvb_ht_class()

        set_values(self, preSigmoidal)
        self.preSigmoidal.value = preSigmoidal
        print_component_summary(self.preSigmoidal.value)


@xai_component(color='rgb(0, 102, 178)')
class Difference(TVBComponent):
    a: InArg[float]

    difference: OutArg[Coupling]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.coupling import Difference
        return Difference

    def execute(self, ctx) -> None:
        difference = self.tvb_ht_class()

        set_values(self, difference)
        self.difference.value = difference
        print_component_summary(self.difference.value)


@xai_component(color='rgb(0, 102, 178)')
class KuramotoCoupling(TVBComponent):
    a: InArg[float]

    kuramoto: OutArg[Coupling]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.coupling import Kuramoto
        return Kuramoto

    def execute(self, ctx) -> None:
        kuramoto = self.tvb_ht_class()

        set_values(self, kuramoto)
        self.kuramoto.value = kuramoto
        print_component_summary(self.kuramoto.value)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#


from xai_components.base import Component
from xai_components.utils import set_defaults


class TVBComponent(Component):

    @property
    def tvb_ht_class(self):
        raise NotImplementedError

    def __init__(self):
        model = self.tvb_ht_class
        set_defaults(self, model)
        del model


class ComponentWithWidget(TVBComponent):
    """
    Used to flag a component that has an associate widget to be displayed in Xircuits UI for interactive setup.
    """



import json
from pathlib import Path

from .handlers import setup_handlers
from ._version import __version__

HERE = Path(__file__).parent.resolve()

with (HERE / "labextension" / "package.json").open() as fid:
    data = json.load(fid)


def _jupyter_labextension_paths():
    return [{"src": "labextension", "dest": data["name"]}]


def _jupyter_server_extension_points():
    return [{"module": "tvbextxircuits"}]


def _load_jupyter_server_extension(server_app):
    """Registers the API handler to receive HTTP requests from the frontend extension.
    Parameters
    ----------
    server_app: jupyterlab.labapp.LabApp
        JupyterLab application instance
    """
    url_path = "xircuits"
    setup_handlers(server_app.web_app, url_path)
    server_app.log.info(
        f"Registered tvbextxircuits extension at URL path /{url_path}"
    )

# For backward compatibility with the classical notebook
load_jupyter_server_extension = _load_jupyter_server_extension


import json
import tornado
from jupyter_server.base.handlers import APIHandler
from .component_parser import ComponentsParser
from tvbextxircuits.nb_generator import NotebookGenerator, WidgetCodeGenerator, ModelConfigLoader

class EditXircuitsFile(APIHandler):
    @tornado.web.authenticated
    def post(self):
        input_data = self.get_json_body()
        xircuits_id = None

        try:
            xircuits_id = input_data["xircuits_id"]
        except KeyError:
            data = {"models_exist": False, "error_msg": "Could not determine the Xircuits ID from POST params!"}
            self.finish(json.dumps(data))

        model_config_loader = ModelConfigLoader(xircuits_id)
        result = model_config_loader.load_configs()

        if result is False:
            data = {"models_exist": False}

        else:
            data = {"models_exist": True,
                    "models": result
                    }

        self.finish(json.dumps(data))


class ComponentsRouteHandler(APIHandler):

    component_parser = ComponentsParser()

    @tornado.web.authenticated
    def get(self):
        self.finish(self.component_parser.get())

    @tornado.web.authenticated
    def post(self):
        input_data = self.get_json_body()

        try:
            component = input_data["component"]
            component_id = input_data["component_id"]
            component_path = input_data["path"]
            xircuits_id = input_data["xircuits_id"]

            notebook_path = self.generate_widget_notebook(component, component_id, component_path, xircuits_id)
            data = {"widget": notebook_path}
            self.finish(json.dumps(data))

        except KeyError:
            data = {"error_msg": "Could not determine the component from POST params!"}
            self.finish(json.dumps(data))

    def generate_widget_notebook(self, component, component_id, component_path, xircuits_id):
        nb_generator = NotebookGenerator()

        text = f"""# Interactive setup for {component} model"""
        nb_generator.add_markdown_cell(text)
        text = f"#### Run the cell below in order to display the Phase Plane\n" \
               f"#### Export the model configuration to add it in the Xircuits diagram\n" \
               f"*Some select fields in the Phase Plane are meant to be disabled in this context"
        nb_generator.add_markdown_cell(text)
        nb_generator.add_code_cell(WidgetCodeGenerator.get_widget_code(component, component_id, component_path))

        path = nb_generator.store(component, xircuits_id)
        return path




# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class ReducedSetFitzHughNagumo(ComponentWithWidget):
    tau: InArg[float]
    a: InArg[float]
    b: InArg[float]
    K11: InArg[float]
    K12: InArg[float]
    K21: InArg[float]
    sigma: InArg[float]
    mu: InArg[float]
    variables_of_interest: InArg[list]

    reducedSetFitzHughNagumo: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.stefanescu_jirsa import ReducedSetFitzHughNagumo
        return ReducedSetFitzHughNagumo

    def execute(self, ctx) -> None:
        reducedSetFitzHughNagumo = self.tvb_ht_class()

        set_values(self, reducedSetFitzHughNagumo)
        self.reducedSetFitzHughNagumo.value = reducedSetFitzHughNagumo
        print_component_summary(self.reducedSetFitzHughNagumo.value)


@xai_component(color='rgb(101, 179, 46)')
class ReducedSetHindmarshRose(ComponentWithWidget):
    r: InArg[float]
    a: InArg[float]
    b: InArg[float]
    c: InArg[float]
    d: InArg[float]
    s: InArg[float]
    xo: InArg[float]
    K11: InArg[float]
    K12: InArg[float]
    K21: InArg[float]
    sigma: InArg[float]
    mu: InArg[float]
    variables_of_interest: InArg[list]

    reducedSetHindmarshRose: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.stefanescu_jirsa import ReducedSetHindmarshRose
        return ReducedSetHindmarshRose

    def execute(self, ctx) -> None:
        reducedSetHindmarshRose = self.tvb_ht_class()

        set_values(self, reducedSetHindmarshRose)
        self.reducedSetHindmarshRose.value = reducedSetHindmarshRose
        print_component_summary(self.reducedSetHindmarshRose.value)



import os
import sys
from datetime import datetime
from urllib.error import HTTPError
import pyunicore.client as unicore_client
from pyunicore.helpers.jobs import Status as unicore_status
from pyunicore.credentials import AuthenticationFailedException
from tvbwidgets.core.auth import get_current_token

import tvbextxircuits._version as xircuits_version
from tvbextxircuits.hpc_config.parse_files import get_files_to_upload


class PyunicoreSubmitter(object):
    storage_name = 'HOME'
    env_dir = 'tvb_xircuits'
    env_name = 'venv'
    modules = {'DAINT-CSCS': 'cray-python', 'JUSUF': 'Python'}
    pip_libraries = 'tvb-ext-xircuits tvb-data'
    EXECUTABLE_KEY = 'Executable'
    PROJECT_KEY = 'Project'
    JOB_TYPE_KEY = 'Job type'
    INTERACTIVE_KEY = 'interactive'

    def __init__(self, site, project):
        self.site = site
        self.project = project

    @property
    def _activate_command(self):
        return f'source ${self.storage_name}/{self.env_dir}/{self.env_name}/bin/activate'

    @property
    def _module_load_command(self):
        return f'module load {self.modules.get(self.site, "")}'

    @property
    def _create_env_command(self):
        return f'cd ${self.storage_name}/{self.env_dir} && rm -rf {self.env_name} && python -mvenv {self.env_name}'

    @property
    def _install_dependencies_command(self):
        return f'pip install -U pip && pip install {self.pip_libraries}'

    def connect_client(self):
        print(f"Connecting to {self.site}...")
        token = get_current_token()
        transport = unicore_client.Transport(token)
        registry = unicore_client.Registry(transport, unicore_client._HBP_REGISTRY_URL)
        sites = registry.site_urls

        try:
            site_url = sites[self.site]
        except KeyError:
            print(f'Site {self.site} seems to be down for the moment.')
            return None

        try:
            client = unicore_client.Client(transport, site_url)
        except (AuthenticationFailedException, HTTPError):
            print(f'Authentication to {self.site} failed, you might not have permissions to access it.', flush=True)
            return None

        print(f'Authenticated to {self.site} with success.', flush=True)
        return client

    def _check_environment_ready(self, home_storage):
        # Pyunicore listdir method returns directory names suffixed by '/'
        if f"{self.env_dir}/" not in home_storage.listdir():
            home_storage.mkdir(self.env_dir)
            print(f"Environment directory not found in HOME, will be created.")
            return False

        if f"{self.env_dir}/{self.env_name}/" not in home_storage.listdir(self.env_dir):
            print(f"Environment not found in HOME, will be created.")
            return False

        try:
            # Check whether tvb-ext-xircuits is installed in HPC env and if version is updated
            site_packages = home_storage.listdir(f'{self.env_dir}/{self.env_name}/lib/python3.9/site-packages')
            files = [file for file in site_packages if "tvb_ext_xircuits" in file]
            assert len(files) >= 1
            remote_version = files[0].split("tvb_ext_xircuits-")[1].split('.dist-info')[0]
            local_version = xircuits_version.__version__
            if remote_version != local_version:
                print(f"Found an older version {remote_version} of tvb-ext-xircuits installed in the environment, "
                      f"will recreate it with {local_version}.")
                return False
            return True
        except HTTPError as e:
            print(f"Could not find site-packages in the environment, will recreate it: {e}")
            return False
        except AssertionError:
            print(f"Could not find tvb-ext-xircuits installed in the environment, will recreate it.")
            return False
        except IndexError:
            print(f"Could not find tvb-ext-xircuits installed in the environment, will recreate it.")
            return False

    def _search_for_home_dir(self, client):
        print(f"Accessing storages on {self.site}...", flush=True)
        num = 10
        offset = 0
        storages = client.get_storages(num=num, offset=offset)
        while len(storages) > 0:
            for storage in storages:
                if storage.resource_url.endswith(self.storage_name):
                    return storage
            offset += num
            storages = client.get_storages(num=num, offset=offset)
        return None

    def _format_date_for_job(self, job):
        date = datetime.strptime(job.properties['submissionTime'], '%Y-%m-%dT%H:%M:%S+%f')
        return date.strftime('%m.%d.%Y, %H:%M:%S')

    def submit_job(self, executable, inputs):
        client = self.connect_client()
        if client is None:
            print(f"Could not connect to {self.site}, stopping execution.")
            return

        home_storage = self._search_for_home_dir(client)
        if home_storage is None:
            print(f"Could not find a {self.storage_name} storage on {self.site}, stopping execution.")
            return

        is_env_ready = self._check_environment_ready(home_storage)

        if is_env_ready:
            print(f"Environment is already prepared, it won't be recreated.")
        else:
            print(f"Preparing environment in your {self.storage_name} folder...", flush=True)
            job_description = {
                self.EXECUTABLE_KEY: f"{self._module_load_command} && {self._create_env_command} && "
                                     f"{self._activate_command} && {self._install_dependencies_command}",
                self.PROJECT_KEY: self.project,
                self.JOB_TYPE_KEY: self.INTERACTIVE_KEY}
            job = client.new_job(job_description, inputs=[])
            print(f"Job is running at {self.site}: {job.working_dir.properties['mountPoint']}. "
                  f"Submission time is: {self._format_date_for_job(job)}. "
                  f"It can be monitored with tvb-ext-unicore.", flush=True)
            job.poll()
            if job.properties['status'] == unicore_status.FAILED:
                print(f"Encountered an error during environment setup, stopping execution.")
                return
            print(f"Successfully finished the environment setup.")

        print("Launching workflow...", flush=True)
        job_description = {
            self.EXECUTABLE_KEY: f"{self._module_load_command} && {self._activate_command} && python {executable}",
            self.PROJECT_KEY: self.project}
        job1 = client.new_job(job_description, inputs=inputs)
        print(f"Job is running at {self.site}: {job1.working_dir.properties['mountPoint']}. "
              f"Submission time is: {self._format_date_for_job(job1)}.", flush=True)
        print('Finished remote launch. Please use tvb-ext-unicore to monitor it.', flush=True)


def get_xircuits_file():
    """
    :return: the file name and the absolute path for the compiled workflow file
    """
    # check that compiled .xircuits file is correctly passed as argument
    file_arg = sys.argv[1]
    print(f'Identified the executable file: {file_arg}', flush=True)

    if os.path.exists(file_arg):
        full_path = os.path.abspath(file_arg)
    else:
        print(f"Cannot find the executable file: {file_arg}", flush=True)
        full_path = None

    filename = os.path.basename(file_arg)

    return filename, full_path


def launch_job(site, project, workflow_file_name, workflow_file_path, files_to_upload):
    """
    Submit a job to a EBRAINS HPC site
    :param site: unicore site
    :param workflow_file_name: base name of compiled workflow file
    :param workflow_file_path: absolute path of compiled workflow file
    :param files_to_upload: list of additional files that need to be sent to the HPC server
    :return: None
    """
    inputs = [workflow_file_path]
    if files_to_upload:
        inputs.extend(files_to_upload)

    PyunicoreSubmitter(site, project).submit_job(workflow_file_name, inputs)


if __name__ == '__main__':
    if len(sys.argv) < 4:
        print(f"Please provide the HPC project to run this job within, stopping execution.")
    else:
        workflow_name, workflow_path = get_xircuits_file()
        print("Preparing job...", flush=True)
        files_to_upload = get_files_to_upload(xircuits_file_path=workflow_path)
        site_arg = sys.argv[2]
        project_arg = sys.argv[3]
        launch_job(site=site_arg, project=project_arg, workflow_file_name=workflow_name,
                   workflow_file_path=workflow_path, files_to_upload=files_to_upload)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class MontbrioPazoRoxin(ComponentWithWidget):
    tau: InArg[float]
    I: InArg[float]
    Delta: InArg[float]
    J: InArg[float]
    eta: InArg[float]
    Gamma: InArg[float]
    cr: InArg[float]
    cv: InArg[float]
    variables_of_interest: InArg[list]
    parameter_names: InArg[list]

    montbrioPazoRoxin: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.infinite_theta import MontbrioPazoRoxin
        return MontbrioPazoRoxin

    def execute(self, ctx) -> None:
        montbrioPazoRoxin = self.tvb_ht_class()

        set_values(self, montbrioPazoRoxin)
        self.montbrioPazoRoxin.value = montbrioPazoRoxin
        print_component_summary(self.montbrioPazoRoxin.value)


@xai_component(color='rgb(101, 179, 46)')
class CoombesByrne(ComponentWithWidget):
    Delta: InArg[float]
    alpha: InArg[float]
    v_syn: InArg[float]
    k: InArg[float]
    eta: InArg[float]
    variables_of_interest: InArg[list]

    coombesByrne: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.infinite_theta import CoombesByrne
        return CoombesByrne

    def execute(self, ctx) -> None:
        coombesByrne = self.tvb_ht_class()

        set_values(self, coombesByrne)
        self.coombesByrne.value = coombesByrne
        print_component_summary(self.coombesByrne.value)


@xai_component(color='rgb(101, 179, 46)')
class CoombesByrne2D(ComponentWithWidget):
    Delta: InArg[float]
    v_syn: InArg[float]
    k: InArg[float]
    eta: InArg[float]
    variables_of_interest: InArg[list]

    coombesByrne2D: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.infinite_theta import CoombesByrne2D
        return CoombesByrne2D

    def execute(self, ctx) -> None:
        coombesByrne2D = self.tvb_ht_class()

        set_values(self, coombesByrne2D)
        self.coombesByrne2D.value = coombesByrne2D
        print_component_summary(self.coombesByrne2D.value)


@xai_component(color='rgb(101, 179, 46)')
class GastSchmidtKnosche_SD(ComponentWithWidget):
    tau: InArg[float]
    tau_A: InArg[float]
    alpha: InArg[float]
    I: InArg[float]
    Delta: InArg[float]
    J: InArg[float]
    eta: InArg[float]
    cr: InArg[float]
    cv: InArg[float]
    variables_of_interest: InArg[list]

    gastSchmidtKnosche_SD: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.infinite_theta import GastSchmidtKnosche_SD
        return GastSchmidtKnosche_SD

    def execute(self, ctx) -> None:
        gastSchmidtKnosche_SD = self.tvb_ht_class()

        set_values(self, gastSchmidtKnosche_SD)
        self.gastSchmidtKnosche_SD.value = gastSchmidtKnosche_SD
        print_component_summary(self.gastSchmidtKnosche_SD.value)


@xai_component(color='rgb(101, 179, 46)')
class GastSchmidtKnosche_SF(ComponentWithWidget):
    tau: InArg[float]
    tau_A: InArg[float]
    alpha: InArg[float]
    I: InArg[float]
    Delta: InArg[float]
    J: InArg[float]
    eta: InArg[float]
    cr: InArg[float]
    cv: InArg[float]
    variables_of_interest: InArg[list]

    gastSchmidtKnosche_SF: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.infinite_theta import GastSchmidtKnosche_SF
        return GastSchmidtKnosche_SF

    def execute(self, ctx) -> None:
        gastSchmidtKnosche_SF = self.tvb_ht_class()

        set_values(self, gastSchmidtKnosche_SF)
        self.gastSchmidtKnosche_SF.value = gastSchmidtKnosche_SF
        print_component_summary(self.gastSchmidtKnosche_SF.value)


@xai_component(color='rgb(101, 179, 46)')
class DumontGutkin(ComponentWithWidget):
    I_e: InArg[float]
    Delta_e: InArg[float]
    eta_e: InArg[float]
    tau_e: InArg[float]
    I_i: InArg[float]
    Delta_i: InArg[float]
    eta_i: InArg[float]
    tau_i: InArg[float]
    tau_s: InArg[float]
    J_ee: InArg[float]
    J_ei: InArg[float]
    J_ie: InArg[float]
    J_ii: InArg[float]
    Gamma: InArg[float]
    variables_of_interest: InArg[list]

    dumontGutkin: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.infinite_theta import DumontGutkin
        return DumontGutkin

    def execute(self, ctx) -> None:
        dumontGutkin = self.tvb_ht_class()

        set_values(self, dumontGutkin)
        self.dumontGutkin.value = dumontGutkin
        print_component_summary(self.dumontGutkin.value)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#
from tvb.simulator.backend.templates import MakoUtilMix
from tvb.datatypes.connectivity import Connectivity
from tvb.datatypes.cortex import Cortex
from tvb.datatypes.patterns import SpatioTemporalPattern
from tvb.simulator.coupling import Coupling
from tvb.simulator.integrators import Integrator
from tvb.simulator.models.base import Model

from xai_components.base import InArg, OutArg, xai_component, InCompArg
from xai_components.base_tvb import TVBComponent
from xai_components.utils import print_component_summary, set_defaults, set_values


@xai_component(color='rgb(220, 5, 45)')
class Simulator(TVBComponent):
    connectivity: InCompArg[Connectivity]
    conduction_speed: InArg[float]
    coupling: InArg[Coupling]
    surface: InArg[Cortex]
    stimulus: InArg[SpatioTemporalPattern]
    model: InArg[Model]
    integrator: InArg[Integrator]
    initial_conditions: InArg[list]
    monitors: InArg[list]
    simulation_length: InArg[float]
    backend: InArg[MakoUtilMix]

    time_series_list: OutArg[list]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)
        self.backend = InArg(None)
        self.time_series = OutArg(None)

    @property
    def tvb_ht_class(self):
        from tvb.simulator.simulator import Simulator
        return Simulator

    def execute(self, ctx) -> None:
        # imports
        from tvb.simulator.backend.nb_mpr import NbMPRBackend

        simulator = self.tvb_ht_class()
        set_values(self, simulator)
        simulator.configure()

        print_component_summary(simulator)

        # run simulation
        backend = self.backend.value
        if isinstance(backend, NbMPRBackend):
            result = backend.run_sim(simulator, simulation_length=simulator.simulation_length)
        else:
            result = simulator.run()

        # create TS
        ts_list = []
        for i in range(len(simulator.monitors)):
            monitor = simulator.monitors[i]
            time, data = result[i]
            ts = monitor.create_time_series(connectivity=simulator.connectivity)
            ts.data = data
            ts.time = time
            ts.configure()
            ts_list.append(ts)

        for ts in ts_list:
            print_component_summary(ts)




# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class LinearModel(ComponentWithWidget):
    gamma: InArg[float]
    variables_of_interest: InArg[list]
    parameter_names: InArg[list]

    linear: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.linear import Linear
        return Linear

    def execute(self, ctx) -> None:
        linear = self.tvb_ht_class()

        set_values(self, linear)
        self.linear.value = linear
        print_component_summary(self.linear.value)



from tvb.datatypes.surfaces import Surface
from xai_components.base import xai_component, InArg, OutArg, InCompArg
from xai_components.base_tvb import TVBComponent
from xai_components.utils import set_values, print_component_summary, set_defaults


@xai_component(color='rgb(0, 116, 92)')
class WhiteMatterSurface(TVBComponent):
    file_path: InArg[str]
    triangle_normals: InArg[list]
    hemisphere_mask: InArg[list]
    zero_based_triangles: InCompArg[bool]

    whiteMatterSurface: OutArg[Surface]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.surfaces import WhiteMatterSurface
        return WhiteMatterSurface

    def __init__(self):
        self.file_path = InArg(None)
        set_defaults(self, self.tvb_ht_class)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'cortex_16384.zip'  # default from tvb_data
        whiteMatterSurface = self.tvb_ht_class.from_file(source_file=file_path)
        set_values(self, whiteMatterSurface)
        whiteMatterSurface.configure()

        self.whiteMatterSurface.value = whiteMatterSurface
        print_component_summary(self.whiteMatterSurface.value)


@xai_component(color='rgb(0, 116, 92)')
class CorticalSurface(TVBComponent):
    file_path: InArg[str]
    triangle_normals: InArg[list]
    hemisphere_mask: InArg[list]
    zero_based_triangles: InCompArg[bool]

    corticalSurface: OutArg[Surface]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.surfaces import CorticalSurface
        return CorticalSurface

    def __init__(self):
        self.file_path = InArg(None)
        set_defaults(self, self.tvb_ht_class)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'cortex_16384.zip'  # default from tvb_data
        corticalSurface = self.tvb_ht_class.from_file(source_file=file_path)
        set_values(self, corticalSurface)
        corticalSurface.configure()

        self.corticalSurface.value = corticalSurface
        print_component_summary(self.corticalSurface.value)


@xai_component(color='rgb(0, 116, 92)')
class SkinAir(TVBComponent):
    file_path: InArg[str]
    triangle_normals: InArg[list]
    hemisphere_mask: InArg[list]
    zero_based_triangles: InCompArg[bool]

    skinAir: OutArg[Surface]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.surfaces import SkinAir
        return SkinAir

    def __init__(self):
        self.file_path = InArg(None)
        set_defaults(self, self.tvb_ht_class)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'outer_skin_4096.zip'  # default from tvb_data
        skinAir = self.tvb_ht_class.from_file(source_file=file_path)
        set_values(self, skinAir)
        skinAir.configure()

        self.skinAir.value = skinAir
        print_component_summary(self.skinAir.value)


@xai_component(color='rgb(0, 116, 92)')
class BrainSkull(TVBComponent):
    file_path: InArg[str]
    triangle_normals: InArg[list]
    hemisphere_mask: InArg[list]
    zero_based_triangles: InCompArg[bool]

    brainSkull: OutArg[Surface]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.surfaces import BrainSkull
        return BrainSkull

    def __init__(self):
        self.file_path = InArg(None)
        set_defaults(self, self.tvb_ht_class)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'inner_skull_4096.zip'  # default from tvb_data
        brainSkull = self.tvb_ht_class.from_file(source_file=file_path)
        set_values(self, brainSkull)
        brainSkull.configure()

        self.brainSkull.value = brainSkull
        print_component_summary(self.brainSkull.value)


@xai_component(color='rgb(0, 116, 92)')
class SkullSkin(TVBComponent):
    file_path: InArg[str]
    triangle_normals: InArg[list]
    hemisphere_mask: InArg[list]
    zero_based_triangles: InCompArg[bool]

    skullSkin: OutArg[Surface]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.surfaces import SkullSkin
        return SkullSkin

    def __init__(self):
        self.file_path = InArg(None)
        set_defaults(self, self.tvb_ht_class)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'outer_skull_4096.zip'  # default from tvb_data
        skullSkin = self.tvb_ht_class.from_file(source_file=file_path)
        set_values(self, skullSkin)
        skullSkin.configure()

        self.skullSkin.value = skullSkin
        print_component_summary(self.skullSkin.value)


@xai_component(color='rgb(0, 116, 92)')
class EEGCap(TVBComponent):
    file_path: InArg[str]
    triangle_normals: InArg[list]
    hemisphere_mask: InArg[list]
    zero_based_triangles: InCompArg[bool]

    eEGCap: OutArg[Surface]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.surfaces import EEGCap
        return EEGCap

    def __init__(self):
        self.file_path = InArg(None)
        set_defaults(self, self.tvb_ht_class)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'scalp_1082.zip'  # default from tvb_data
        eEGCap = self.tvb_ht_class.from_file(source_file=file_path)
        set_values(self, eEGCap)
        eEGCap.configure()

        self.eEGCap.value = eEGCap
        print_component_summary(self.eEGCap.value)


@xai_component(color='rgb(0, 116, 92)')
class FaceSurface(TVBComponent):
    file_path: InArg[str]
    triangle_normals: InArg[list]
    hemisphere_mask: InArg[list]
    zero_based_triangles: InCompArg[bool]

    faceSurface: OutArg[Surface]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.surfaces import FaceSurface
        return FaceSurface

    def __init__(self):
        self.file_path = InArg(None)
        set_defaults(self, self.tvb_ht_class)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'face_8614.zip'  # default from tvb_data
        faceSurface = self.tvb_ht_class.from_file(source_file=file_path)
        set_values(self, faceSurface)
        faceSurface.configure()

        self.faceSurface.value = faceSurface
        print_component_summary(self.faceSurface.value)



from tvb.datatypes.projections import ProjectionMatrix
from tvb.datatypes.sensors import Sensors
from tvb.datatypes.surfaces import Surface
from xai_components.base import xai_component, InArg, OutArg, Component, InCompArg
from xai_components.utils import print_component_summary


@xai_component(color='rgb(67, 47, 106)')
class ProjectionSurfaceEEG(Component):
    from tvb.datatypes.projections import ProjectionSurfaceEEG
    file_path: InArg[str]
    sources: InCompArg[Surface]
    sensors: InCompArg[Sensors]

    projectionSurfaceEEG: OutArg[ProjectionMatrix]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.projections import ProjectionSurfaceEEG
        return ProjectionSurfaceEEG

    def __init__(self):
        self.done = False

        self.file_path = InArg(None)
        self.sources = InCompArg(None)
        self.sensors = InCompArg(None)
        self.projectionSurfaceEEG = OutArg(None)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'projection_eeg_65_surface_16k.npy'  # default from tvb_data
        projectionSurfaceEEG = self.ProjectionSurfaceEEG.from_file(source_file=file_path)

        projectionSurfaceEEG.sources = self.sources.value
        projectionSurfaceEEG.sensors = self.sensors.value
        projectionSurfaceEEG.configure()

        self.projectionSurfaceEEG.value = projectionSurfaceEEG
        print_component_summary(self.projectionSurfaceEEG.value)


@xai_component(color='rgb(67, 47, 106)')
class ProjectionSurfaceMEG(Component):
    file_path: InArg[str]
    sources: InCompArg[Surface]
    sensors: InCompArg[Sensors]

    projectionSurfaceMEG: OutArg[ProjectionMatrix]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.projections import ProjectionSurfaceMEG
        return ProjectionSurfaceMEG

    def __init__(self):
        self.done = False

        self.file_path = InArg(None)
        self.sources = InCompArg(None)
        self.sensors = InCompArg(None)
        self.projectionSurfaceMEG = OutArg(None)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'projection_meg_276_surface_16k.npy'  # default from tvb_data
        projectionSurfaceMEG = self.tvb_ht_class.from_file(source_file=file_path)

        projectionSurfaceMEG.sources = self.sources.value
        projectionSurfaceMEG.sensors = self.sensors.value
        projectionSurfaceMEG.configure()

        self.projectionSurfaceMEG.value = projectionSurfaceMEG
        print_component_summary(self.projectionSurfaceMEG.value)


@xai_component(color='rgb(67, 47, 106)')
class ProjectionSurfaceSEEG(Component):
    file_path: InArg[str]
    sources: InCompArg[Surface]
    sensors: InCompArg[Sensors]

    projectionSurfaceSEEG: OutArg[ProjectionMatrix]

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.projections import ProjectionSurfaceSEEG
        return ProjectionSurfaceSEEG

    def __init__(self):
        self.done = False

        self.file_path = InArg(None)
        self.sources = InCompArg(None)
        self.sensors = InCompArg(None)
        self.projectionSurfaceSEEG = OutArg(None)

    def execute(self, ctx) -> None:
        file_path = self.file_path.value
        if not file_path:
            file_path = 'projection_seeg_588_surface_16k.npy'  # default from tvb_data
        projectionSurfaceSEEG = self.tvb_ht_class.from_file(source_file=file_path)

        projectionSurfaceSEEG.sources = self.sources.value
        projectionSurfaceSEEG.sensors = self.sensors.value
        projectionSurfaceSEEG.configure()

        self.projectionSurfaceSEEG.value = projectionSurfaceSEEG
        print_component_summary(self.projectionSurfaceSEEG.value)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


# TODO: uncomment once KuramotoT is fixed in tvb-library
# @xai_component(color='rgb(101, 179, 46)')
# class KuramotoT(ComponentWithWidget):
#     omega: InArg[float]
#     variables_of_interest: InArg[list]
#
#     kuramotoT: OutArg[Model]
#
#     @property
#     def tvb_ht_class(self):
#         from tvb.simulator.models.kuramotoT import KuramotoT
#         return KuramotoT
#
#     def execute(self, ctx) -> None:
#         kuramotoT = self.tvb_ht_class()
#
#         set_values(self, kuramotoT)
#         self.kuramotoT.value = kuramotoT
#         print_component_summary(self.kuramotoT.value)



"""
tvbextxircuits setup
"""
import json
import sys
from pathlib import Path

import setuptools
from setuptools import setup, find_packages

import os

HERE = Path(__file__).parent.resolve()

# The name of the project
name = "tvb-ext-xircuits"

lab_path = (HERE / "tvbextxircuits" / "labextension")

# Representative files that should exist after a successful build
ensured_targets = [
    str(lab_path / "package.json"),
    str(lab_path / "static/style.js")
]

labext_name = "tvb-ext-xircuits"

data_files_spec = [
    ("share/jupyter/labextensions/%s" % labext_name, str(lab_path.relative_to(HERE)), "**"),
    ("share/jupyter/labextensions/%s" % labext_name, str("."), "install.json"),
    ("etc/jupyter/jupyter_server_config.d",
     "jupyter-config/server-config", "xircuits.json"),
    # For backward compatibility with notebook server
    ("etc/jupyter/jupyter_notebook_config.d",
     "jupyter-config/nb-config", "xircuits.json"),
]

long_description = (HERE / "README.md").read_text()

# Get the package info from package.json
pkg_json = json.loads((HERE / "package.json").read_bytes())
version = (
    pkg_json["version"]
    .replace("-alpha.", "a")
    .replace("-beta.", "b")
    .replace("-rc.", "rc")
)

with open("requirements.txt", encoding='utf-8-sig') as f:
    required = f.read().splitlines()

component_library_reqs = {}
full_reqs = []

for lib_name in os.listdir("xai_components/"):

    req_path = "xai_components/" + lib_name + "/requirements.txt"
    
    if os.path.exists(req_path):
        with open(req_path, encoding='utf-8-sig') as f:
            packages_required = f.read().splitlines()
            package_name = "_".join(lib_name.split("_")[1:])
            component_library_reqs.update({package_name : packages_required})
            full_reqs.extend(component_library_reqs)

component_library_reqs.update({"full": full_reqs})

setup_args = dict(
    name=name,
    version=version,
    url=pkg_json["homepage"],
    author=pkg_json["author"]["name"],
    author_email=pkg_json["author"]["email"],
    description=pkg_json["description"],
    license=pkg_json["license"],
    license_file="LICENSE",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    install_requires=required,
    extras_require=component_library_reqs,
    zip_safe=False,
    include_package_data=True,
    python_requires=">=3.8",
    platforms="Linux, Mac OS X, Windows",
    keywords=["Jupyter", "JupyterLab", "JupyterLab3"],
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Framework :: Jupyter",
        "Framework :: Jupyter :: JupyterLab",
        "Framework :: Jupyter :: JupyterLab :: 3",
        "Framework :: Jupyter :: JupyterLab :: Extensions",
        "Framework :: Jupyter :: JupyterLab :: Extensions :: Prebuilt",
    ],
    entry_points ={
        'console_scripts': [
            'tvbextxircuits = tvbextxircuits.start_xircuits:main',
            'tvbextxircuits-examples = tvbextxircuits.start_xircuits:download_examples',
            'tvbextxircuits-components = tvbextxircuits.start_xircuits:download_component_library'
            ]}
)

try:
    from jupyter_packaging import (
        wrap_installers,
        npm_builder,
        get_data_files
    )
    post_develop = npm_builder(
        build_cmd="install:extension", source_dir="src", build_dir=lab_path
    )
    setup_args["cmdclass"] = wrap_installers(post_develop=post_develop, ensured_targets=ensured_targets)
    setup_args["data_files"] = get_data_files(data_files_spec)
except ImportError as e:
    import logging
    logging.basicConfig(format="%(levelname)s: %(message)s")
    logging.warning("Build tool `jupyter-packaging` is missing. Install it with pip or conda.")
    if not ("--name" in sys.argv or "--version" in sys.argv):
        raise e

if __name__ == "__main__":
    setuptools.setup(**setup_args)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.datatypes.equations import Equation

from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import TVBComponent
from xai_components.utils import print_component_summary, set_defaults, set_values


@xai_component(color='rgb(253, 225, 0)')
class DiscreteEquation(TVBComponent):
    parameters: InArg[dict]

    discreteEquation: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import DiscreteEquation
        return DiscreteEquation

    def execute(self, ctx) -> None:
        discreteEquation = self.tvb_ht_class()

        set_values(self, discreteEquation)
        self.discreteEquation.value = discreteEquation
        print_component_summary(self.discreteEquation.value)


@xai_component(color='rgb(253, 225, 0)')
class LinearEquation(TVBComponent):
    parameters: InArg[dict]

    linear: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import Linear
        return Linear

    def execute(self, ctx) -> None:
        linear = self.tvb_ht_class()

        set_values(self, linear)
        self.linear.value = linear
        print_component_summary(self.linear.value)


@xai_component(color='rgb(253, 225, 0)')
class Gaussian(TVBComponent):
    parameters: InArg[dict]

    gaussian: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import Gaussian
        return Gaussian

    def execute(self, ctx) -> None:
        gaussian = self.tvb_ht_class()

        set_values(self, gaussian)
        self.gaussian.value = gaussian
        print_component_summary(self.gaussian.value)


@xai_component(color='rgb(253, 225, 0)')
class DoubleGaussian(TVBComponent):
    parameters: InArg[dict]

    doubleGaussian: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import DoubleGaussian
        return DoubleGaussian

    def execute(self, ctx) -> None:
        doubleGaussian = self.tvb_ht_class()

        set_values(self, doubleGaussian)
        self.doubleGaussian.value = doubleGaussian
        print_component_summary(self.doubleGaussian.value)


@xai_component(color='rgb(253, 225, 0)')
class Sigmoid(TVBComponent):
    parameters: InArg[dict]

    sigmoid: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import Sigmoid
        return Sigmoid

    def execute(self, ctx) -> None:
        sigmoid = self.tvb_ht_class()

        set_values(self, sigmoid)
        self.sigmoid.value = sigmoid
        print_component_summary(self.sigmoid.value)


@xai_component(color='rgb(253, 225, 0)')
class GeneralizedSigmoid(TVBComponent):
    parameters: InArg[dict]

    generalizedSigmoid: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import GeneralizedSigmoid
        return GeneralizedSigmoid

    def execute(self, ctx) -> None:
        generalizedSigmoid = self.tvb_ht_class()

        set_values(self, generalizedSigmoid)
        self.generalizedSigmoid.value = generalizedSigmoid
        print_component_summary(self.generalizedSigmoid.value)


@xai_component(color='rgb(253, 225, 0)')
class Sinusoid(TVBComponent):
    parameters: InArg[dict]

    sinusoid: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import Sinusoid
        return Sinusoid

    def execute(self, ctx) -> None:
        sinusoid = self.tvb_ht_class()

        set_values(self, sinusoid)
        self.sinusoid.value = sinusoid
        print_component_summary(self.sinusoid.value)


@xai_component(color='rgb(253, 225, 0)')
class Cosine(TVBComponent):
    parameters: InArg[dict]

    cosine: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import Cosine
        return Cosine

    def execute(self, ctx) -> None:
        cosine = self.tvb_ht_class()

        set_values(self, cosine)
        self.cosine.value = cosine
        print_component_summary(self.cosine.value)


@xai_component(color='rgb(253, 225, 0)')
class Alpha(TVBComponent):
    parameters: InArg[dict]

    alpha: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import Alpha
        return Alpha

    def execute(self, ctx) -> None:
        alpha = self.tvb_ht_class()

        set_values(self, alpha)
        self.alpha.value = alpha
        print_component_summary(self.alpha.value)


@xai_component(color='rgb(253, 225, 0)')
class PulseTrain(TVBComponent):
    parameters: InArg[dict]

    pulseTrain: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import PulseTrain
        return PulseTrain


    def execute(self, ctx) -> None:
        pulseTrain = self.tvb_ht_class()

        set_values(self, pulseTrain)
        self.pulseTrain.value = pulseTrain
        print_component_summary(self.pulseTrain.value)


@xai_component(color='rgb(253, 225, 0)')
class Gamma(TVBComponent):
    parameters: InArg[dict]

    gamma: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import Gamma
        return Gamma

    def execute(self, ctx) -> None:
        gamma = self.tvb_ht_class()

        set_values(self, gamma)
        self.gamma.value = gamma
        print_component_summary(self.gamma.value)


@xai_component(color='rgb(253, 225, 0)')
class DoubleExponential(TVBComponent):
    parameters: InArg[dict]

    doubleExponential: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import DoubleExponential
        return DoubleExponential


    def execute(self, ctx) -> None:
        doubleExponential = self.tvb_ht_class()

        set_values(self, doubleExponential)
        self.doubleExponential.value = doubleExponential
        print_component_summary(self.doubleExponential.value)


@xai_component(color='rgb(253, 225, 0)')
class FirstOrderVolterra(TVBComponent):
    parameters: InArg[dict]

    firstOrderVolterra: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import FirstOrderVolterra
        return FirstOrderVolterra

    def execute(self, ctx) -> None:
        firstOrderVolterra = self.tvb_ht_class()

        set_values(self, firstOrderVolterra)
        self.firstOrderVolterra.value = firstOrderVolterra
        print_component_summary(self.firstOrderVolterra.value)


@xai_component(color='rgb(253, 225, 0)')
class MixtureOfGammas(TVBComponent):
    parameters: InArg[dict]

    mixtureOfGammas: OutArg[Equation]

    def __init__(self):
        set_defaults(self, self.tvb_ht_class)

    @property
    def tvb_ht_class(self):
        from tvb.datatypes.equations import MixtureOfGammas
        return MixtureOfGammas

    def execute(self, ctx) -> None:
        mixtureOfGammas = self.tvb_ht_class()

        set_values(self, mixtureOfGammas)
        self.mixtureOfGammas.value = mixtureOfGammas
        print_component_summary(self.mixtureOfGammas.value)



from argparse import Namespace
from typing import TypeVar, Generic

T = TypeVar('T')


class InArg(Generic[T]):
    value: T

    def __init__(self, value: T) -> None:
        self.value = value

    @classmethod
    def empty(cls):
        return InArg(None)


class OutArg(Generic[T]):
    value: T

    def __init__(self, value: T) -> None:
        self.value = value

    @classmethod
    def empty(cls):
        return OutArg(None)

class InCompArg(Generic[T]):
    value: T

    def __init__(self, value: T) -> None:
        self.value = value

    @classmethod
    def empty(cls):
        return InCompArg(None)


def xai_component(*args, **kwargs):
    # Passthrough element without any changes.
    # This is used for parser metadata only.
    if len(args) == 1 and callable(args[0]):
        # @xai_components form
        return args[0]
    else:
        # @xai_components(...) form
        def passthrough(f):
            return f
        return passthrough

class ExecutionContext:
    args: Namespace

    def __init__(self, args: Namespace):
        self.args = args


class BaseComponent:
    @classmethod
    def set_execution_context(cls, context: ExecutionContext) -> None:
        cls.execution_context = context

    def execute(self) -> None:
        pass

    def do(self):
        pass


class Component(BaseComponent):
    next: BaseComponent
    done: False

    def do(self, ctx) -> BaseComponent:
        print(f"\nExecuting: {self.__class__.__name__}")
        self.execute(ctx)

        return self.done, self.next

    def debug_repr(self) -> str:
        return "<h1>Component</h1>"


class BranchComponent(BaseComponent):
    when_true: BaseComponent
    when_false: BaseComponent

    condition: InArg[bool]

    def do(self, ctx) -> BaseComponent:
        if self.condition.value:
            return self.when_true
        else:
            return self.when_false


@xai_component
class LoopComponent(Component):
    body: Component

    condition: InArg[bool]

    def do(self, ctx) -> BaseComponent:
        while self.condition.value:
            next_body = self.body.do(ctx)
            while next_body:
                next_body = next_body.do(ctx)
            return self
        return self.next


def execute_graph(args: Namespace, start: BaseComponent, ctx) -> None:
    BaseComponent.set_execution_context(ExecutionContext(args))

    if 'debug' in args and args['debug']:
        import pdb
        pdb.set_trace()

        current_component = start
        next_component = current_component.do(ctx)
        while next_component:
            current_component = next_component
            next_component = current_component.do(ctx)
    else:
        next_component = start.do(ctx)
        while next_component:
            next_component = next_component.do(ctx)



# -*- coding: utf-8 -*-
#
# "TheVirtualBrain - Widgets" package
#
# (c) 2022-2023, TVB Widgets Team
#

from tvb.simulator.models.base import Model
from xai_components.base import xai_component, InArg, OutArg
from xai_components.base_tvb import ComponentWithWidget
from xai_components.utils import print_component_summary, set_values


@xai_component(color='rgb(101, 179, 46)')
class ReducedWongWangExcInh(ComponentWithWidget):
    a_e: InArg[float]
    b_e: InArg[float]
    d_e: InArg[float]
    gamma_e: InArg[float]
    tau_e: InArg[float]
    w_p: InArg[float]
    J_N: InArg[float]
    W_e: InArg[float]
    a_i: InArg[float]
    b_i: InArg[float]
    d_i: InArg[float]
    gamma_i: InArg[float]
    tau_i: InArg[float]
    J_i: InArg[float]
    W_i: InArg[float]
    I_o: InArg[float]
    I_ext: InArg[float]
    G: InArg[float]
    lamda: InArg[float]
    variables_of_interest: InArg[list]

    reducedWongWangExcInh: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.wong_wang_exc_inh import ReducedWongWangExcInh
        return ReducedWongWangExcInh

    def execute(self, ctx) -> None:
        reducedWongWangExcInh = self.tvb_ht_class()

        set_values(self, reducedWongWangExcInh)
        self.reducedWongWangExcInh.value = reducedWongWangExcInh
        print_component_summary(self.reducedWongWangExcInh.value)


@xai_component(color='rgb(101, 179, 46)')
class DecoBalancedExcInh(ComponentWithWidget):
    M_i: InArg[float]
    a_e: InArg[float]
    b_e: InArg[float]
    d_e: InArg[float]
    gamma_e: InArg[float]
    tau_e: InArg[float]
    w_p: InArg[float]
    J_N: InArg[float]
    W_e: InArg[float]
    a_i: InArg[float]
    b_i: InArg[float]
    d_i: InArg[float]
    gamma_i: InArg[float]
    tau_i: InArg[float]
    J_i: InArg[float]
    W_i: InArg[float]
    I_o: InArg[float]
    I_ext: InArg[float]
    G: InArg[float]
    lamda: InArg[float]
    variables_of_interest: InArg[list]

    decoBalancedExcInh: OutArg[Model]

    @property
    def tvb_ht_class(self):
        from tvb.simulator.models.wong_wang_exc_inh import DecoBalancedExcInh
        return DecoBalancedExcInh

    def execute(self, ctx) -> None:
        decoBalancedExcInh = self.tvb_ht_class()

        set_values(self, decoBalancedExcInh)
        self.decoBalancedExcInh.value = decoBalancedExcInh
        print_component_summary(self.decoBalancedExcInh.value)



import json
import os.path
import shutil

from tvbextxircuits import nb_generator
from tvbextxircuits.nb_generator import ModelConfigLoader

SUP_HOPF_DEFAULT_PARAMS = {
    'a': [-0.5],
    'omega': [1],
    'model': 'SupHopf'
}

SUP_HOPF_DEFAULT_PARAMS2 = {
    'a': [0.5],
    'omega': [1],
    'model': 'SupHopf2'
}

TEMP_DIR = "temp_test_dir"


def setup_function():
    os.mkdir(TEMP_DIR)


def test_model_config_loader_false():
    nb_generator.NOTEBOOKS_DIR = TEMP_DIR

    dummy_xircuits_id = "abc-def-ghi-jkl"
    model_config_loader = ModelConfigLoader(dummy_xircuits_id)

    # Test when no xircuits_id folder exists
    result = model_config_loader.load_configs()
    assert result is False

    xircuits_dir = os.path.join(TEMP_DIR, dummy_xircuits_id)
    os.mkdir(xircuits_dir)
    model_config_loader = ModelConfigLoader(dummy_xircuits_id)

    # Test when no model config file exists
    result = model_config_loader.load_configs()
    assert result is False


def create_xircuits_dir():
    dummy_xircuits_id = "abc-def-ghi-jkl"
    xircuits_dir = os.path.join(TEMP_DIR, dummy_xircuits_id)
    os.mkdir(xircuits_dir)
    return xircuits_dir, dummy_xircuits_id


def create_model_config_file(xircuits_dir, model_id=1):
    dummy_model_id = f"abc-def-ghi{model_id}"
    model_config_filename = f"model_{dummy_model_id}.json"
    model_config_file = os.path.join(xircuits_dir, model_config_filename)
    model = SUP_HOPF_DEFAULT_PARAMS
    if model_id == 2:
        model = SUP_HOPF_DEFAULT_PARAMS2
    with open(model_config_file, 'w') as f:
        json.dump({'model': model}, f)
    return dummy_model_id


def test_model_config_loader_single_file():
    xircuits_dir, dummy_xircuits_id = create_xircuits_dir()
    model_config_loader = ModelConfigLoader(dummy_xircuits_id)

    dummy_model_id = create_model_config_file(xircuits_dir)

    result = model_config_loader.load_configs()
    assert type(result) is dict
    assert len(result) == 1
    result_id = list(result.values())[0]['id']
    assert result_id == dummy_model_id
    result_params_a = list(result.values())[0]['params']['a']
    assert result_params_a['name'] == 'a'
    assert result_params_a['value'] == SUP_HOPF_DEFAULT_PARAMS['a'][0]
    assert result_params_a['type'] == 'float'
    assert len(os.listdir(TEMP_DIR)) == 0


def test_model_config_loader_multiple_files():
    xircuits_dir, dummy_xircuits_id = create_xircuits_dir()
    model_config_loader = ModelConfigLoader(dummy_xircuits_id)

    dummy_model_id1 = create_model_config_file(xircuits_dir)
    dummy_model_id2 = create_model_config_file(xircuits_dir, 2)

    result = model_config_loader.load_configs()
    assert type(result) is dict
    assert len(result) == 2
    assert len(os.listdir(TEMP_DIR)) == 0

    def asserts(result_id, dummy_model_id, result_params, default_params):
        assert result_id == dummy_model_id
        assert result_params['a']['name'] == 'a'
        assert result_params['a']['value'] == default_params['a'][0]
        assert result_params['a']['type'] == 'float'

    result_id1 = list(result.values())[0]['id']
    result_params1 = list(result.values())[0]['params']
    result_id2 = list(result.values())[1]['id']
    result_params2 = list(result.values())[1]['params']

    if result_id1 == dummy_model_id1:
        asserts(result_id1, dummy_model_id1, result_params1, SUP_HOPF_DEFAULT_PARAMS)
        asserts(result_id2, dummy_model_id2, result_params2, SUP_HOPF_DEFAULT_PARAMS2)
    else:
        asserts(result_id1, dummy_model_id2, result_params1, SUP_HOPF_DEFAULT_PARAMS2)
        asserts(result_id2, dummy_model_id1, result_params2, SUP_HOPF_DEFAULT_PARAMS)


def teardown_function():
    shutil.rmtree(TEMP_DIR)
